@@ -14,59 +14,30 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
     As a special exception, permission is given to link this program
     with any edition of Qt, and distribute the resulting executable,
     without including the source code for Qt in the source distribution.
 */
-
 #ifndef KCMKABCUSTOMFIELDS_H
 #define KCMKABCUSTOMFIELDS_H
 
-#include <kcmodule.h>
+#include <libkdepim/kcmdesignerfields.h>
 
-class KAboutData;
-class KListView;
-
-class QLabel;
-
-class KCMKabCustomFields : public KCModule
+class KCMKabCustomFields : public KPIM::KCMDesignerFields
 {
-  Q_OBJECT
-
   public:
     KCMKabCustomFields( QWidget *parent = 0, const char *name = 0 );
 
-    virtual void load();
-    virtual void save();
-    virtual void defaults();
-    virtual const KAboutData* aboutData() const;
-
   protected:
-    void        loadUiFiles();
-    void        loadActivePages(const QStringList&);
-    QStringList saveActivePages();
-    QString     kabLocalDir();
+    QString localUiDir();
+    QString uiPath();
+    void writeActivePages( const QStringList & );
+    QStringList readActivePages();
+    QString applicationName();
 
-  private slots:
-    void updatePreview( QListViewItem* );
-    void itemClicked( QListViewItem* );
-    void startDesigner();
-    void rebuildList();
-    void deleteFile();
-    void importFile();
-
-    
-  private:
-    void initGUI();
-
-    KListView *mPageView;
-    QLabel *mPagePreview;
-    QLabel *mPageDetails;
-    QPushButton *mDeleteButton;    
-    QPushButton *mImportButton;
-    QPushButton *mDesignerButton;
+    QString kabLocalDir();
 };
 
 #endif
