@@ -1,7 +1,7 @@
 /*
  *  messagewin.cpp  -  displays an alarm message
  *  Program:  kalarm
- *  (C) 2001 - 2005 by David Jarvie <software@astrojar.org.uk>
+ *  Copyright © 2001-2007 by David Jarvie <software@astrojar.org.uk>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -13,9 +13,9 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  *  GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  You should have received a copy of the GNU General Public License along
+ *  with this program; if not, write to the Free Software Foundation, Inc.,
+ *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "kalarm.h"
@@ -31,7 +31,6 @@
 #include <qwhatsthis.h>
 #include <qtooltip.h>
 #include <qdragobject.h>
-#define KDE2_QTEXTEDIT_VIEW     // for KDE2 QTextEdit compatibility
 #include <qtextedit.h>
 #include <qtimer.h>
 
@@ -44,6 +43,7 @@
 #include <kiconloader.h>
 #include <kdialog.h>
 #include <ktextbrowser.h>
+#include <kglobalsettings.h>
 #include <kmimetype.h>
 #include <kmessagebox.h>
 #include <kwin.h>
@@ -59,12 +59,13 @@
 #include <arts/kartsserver.h>
 #include <arts/kplayobjectfactory.h>
 #include <arts/kplayobject.h>
-#include <dcopclient.h>
 #endif
+#include <dcopclient.h>
 #include <kdebug.h>
 
 #include "alarmcalendar.h"
 #include "deferdlg.h"
+#include "editdlg.h"
 #include "functions.h"
 #include "kalarmapp.h"
 #include "mainwindow.h"
@@ -79,6 +80,7 @@ static const char* KMIX_APP_NAME    = "kmix";
 static const char* KMIX_DCOP_OBJECT = "Mixer0";
 static const char* KMIX_DCOP_WINDOW = "kmix-mainwindow#1";
 #endif
+static const char* KMAIL_DCOP_OBJECT = "KMailIface";
 
 // The delay for enabling message window buttons if a zero delay is
 // configured, i.e. the windows are placed far from the cursor.
@@ -121,8 +123,16 @@ class MWMimeSourceFactory : public QMimeSourceFactory
 // Basic flags for the window
 static const Qt::WFlags WFLAGS = Qt::WStyle_StaysOnTop | Qt::WDestructiveClose;
 
+// Error message bit masks
+enum {
+	ErrMsg_Speak     = 0x01,
+	ErrMsg_AudioFile = 0x02,
+	ErrMsg_Volume    = 0x04
+};
 
-QPtrList<MessageWin> MessageWin::mWindowList;
+
+QValueList<MessageWin*> MessageWin::mWindowList;
+QMap<QString, unsigned> MessageWin::mErrorMessages;
 
 
 /******************************************************************************
@@ -133,26 +143,32 @@ QPtrList<MessageWin> MessageWin::mWindowList;
 */
 MessageWin::MessageWin(const KAEvent& event, const KAAlarm& alarm, bool reschedule_event, bool allowDefer)
 	: MainWindowBase(0, "MessageWin", WFLAGS | Qt::WGroupLeader | Qt::WStyle_ContextHelp
-	                                         | (Preferences::instance()->modalMessages() ? 0 : Qt::WX11BypassWM)),
+	                                         | (Preferences::modalMessages() ? 0 : Qt::WX11BypassWM)),
 	  mMessage(event.cleanText()),
 	  mFont(event.font()),
 	  mBgColour(event.bgColour()),
 	  mFgColour(event.fgColour()),
-	  mDateTime((alarm.type() & KAAlarm::REMINDER_ALARM) ? event.mainDateTime() : alarm.dateTime()),
+	  mDateTime((alarm.type() & KAAlarm::REMINDER_ALARM) ? event.mainDateTime(true) : alarm.dateTime(true)),
 	  mEventID(event.id()),
 	  mAudioFile(event.audioFile()),
 	  mVolume(event.soundVolume()),
+	  mFadeVolume(event.fadeVolume()),
+	  mFadeSeconds(QMIN(event.fadeSeconds(), 86400)),
+	  mDefaultDeferMinutes(event.deferDefaultMinutes()),
 	  mAlarmType(alarm.type()),
 	  mAction(event.action()),
+	  mKMailSerialNumber(event.kmailSerialNumber()),
 	  mRestoreHeight(0),
 	  mAudioRepeat(event.repeatSound()),
 	  mConfirmAck(event.confirmAck()),
+	  mShowEdit(!mEventID.isEmpty()),
 	  mNoDefer(!allowDefer || alarm.repeatAtLogin()),
 	  mInvalid(false),
 	  mArtsDispatcher(0),
 	  mPlayObject(0),
 	  mOldVolume(-1),
 	  mEvent(event),
+	  mEditButton(0),
 	  mDeferButton(0),
 	  mSilenceButton(0),
 	  mDeferDlg(0),
@@ -160,13 +176,15 @@ MessageWin::MessageWin(const KAEvent& event, const KAAlarm& alarm, bool reschedu
 	  mFlags(event.flags()),
 	  mLateCancel(event.lateCancel()),
 	  mErrorWindow(false),
-	  mNoPostAction(false),
+	  mNoPostAction(alarm.type() & KAAlarm::REMINDER_ALARM),
 	  mRecreating(false),
 	  mBeep(event.beep()),
+	  mSpeak(event.speak()),
 	  mRescheduleEvent(reschedule_event),
 	  mShown(false),
 	  mPositioning(false),
-	  mDeferClosing(false)
+	  mNoCloseConfirm(false),
+	  mDisableDeferral(false)
 {
 	kdDebug(5950) << "MessageWin::MessageWin(event)" << endl;
 	// Set to save settings automatically, but don't save window size.
@@ -174,6 +192,8 @@ MessageWin::MessageWin(const KAEvent& event, const KAAlarm& alarm, bool reschedu
 	setAutoSaveSettings(QString::fromLatin1("MessageWin"), false);
 	initView();
 	mWindowList.append(this);
+	if (event.autoClose())
+		mCloseTime = alarm.dateTime().dateTime().addSecs(event.lateCancel() * 60);
 }
 
 /******************************************************************************
@@ -186,14 +206,17 @@ MessageWin::MessageWin(const KAEvent& event, const DateTime& alarmDateTime, cons
 	  mEventID(event.id()),
 	  mAlarmType(KAAlarm::MAIN_ALARM),
 	  mAction(event.action()),
+	  mKMailSerialNumber(0),
 	  mErrorMsgs(errmsgs),
 	  mRestoreHeight(0),
 	  mConfirmAck(false),
+	  mShowEdit(false),
 	  mNoDefer(true),
 	  mInvalid(false),
 	  mArtsDispatcher(0),
 	  mPlayObject(0),
 	  mEvent(event),
+	  mEditButton(0),
 	  mDeferButton(0),
 	  mSilenceButton(0),
 	  mDeferDlg(0),
@@ -204,7 +227,8 @@ MessageWin::MessageWin(const KAEvent& event, const DateTime& alarmDateTime, cons
 	  mRescheduleEvent(false),
 	  mShown(false),
 	  mPositioning(false),
-	  mDeferClosing(false)
+	  mNoCloseConfirm(false),
+	  mDisableDeferral(false)
 {
 	kdDebug(5950) << "MessageWin::MessageWin(errmsg)" << endl;
 	initView();
@@ -219,18 +243,20 @@ MessageWin::MessageWin()
 	: MainWindowBase(0, "MessageWin", WFLAGS),
 	  mArtsDispatcher(0),
 	  mPlayObject(0),
+	  mEditButton(0),
+	  mDeferButton(0),
 	  mSilenceButton(0),
 	  mDeferDlg(0),
 	  mWinModule(0),
 	  mErrorWindow(false),
-	  mNoPostAction(true),
 	  mRecreating(false),
 	  mRescheduleEvent(false),
 	  mShown(false),
 	  mPositioning(false),
-	  mDeferClosing(false)
+	  mNoCloseConfirm(false),
+	  mDisableDeferral(false)
 {
-	kdDebug(5950) << "MessageWin::MessageWin()\n";
+	kdDebug(5950) << "MessageWin::MessageWin(restore)\n";
 	mWindowList.append(this);
 }
 
@@ -239,16 +265,12 @@ MessageWin::MessageWin()
 */
 MessageWin::~MessageWin()
 {
-	kdDebug(5950) << "MessageWin::~MessageWin()\n";
+	kdDebug(5950) << "MessageWin::~MessageWin(" << mEventID << ")" << endl;
 	stopPlay();
 	delete mWinModule;
 	mWinModule = 0;
-	for (MessageWin* w = mWindowList.first();  w;  w = mWindowList.next())
-		if (w == this)
-		{
-			mWindowList.remove();
-			break;
-		}
+	mErrorMessages.remove(mEventID);
+	mWindowList.remove(this);
 	if (!mRecreating)
 	{
 		if (!mNoPostAction  &&  !mEvent.postAction().isEmpty())
@@ -370,8 +392,8 @@ void MessageWin::initView()
 				text->setMinimumHeight(QMIN(h, lineSpacing*4));
 				text->setMaximumWidth(s.width() + text->scrollBarWidth());
 				QWhatsThis::add(text, i18n("The alarm message"));
-				int vspace = lineSpacing/2 - marginKDE2;
-				int hspace = lineSpacing - marginKDE2 - KDialog::marginHint();
+				int vspace = lineSpacing/2;
+				int hspace = lineSpacing - KDialog::marginHint();
 				topLayout->addSpacing(vspace);
 				topLayout->addStretch();
 				// Don't include any horizontal margins if message is 2/3 screen width
@@ -405,7 +427,7 @@ void MessageWin::initView()
 			if (mDateTime.isDateOnly()  ||  QDate::currentDate().daysTo(mDateTime.date()) > 0)
 			{
 				setRemainingTextDay();
-				DailyTimer::connect(this, SLOT(setRemainingTextDay()));    // update every day
+				MidnightTimer::connect(this, SLOT(setRemainingTextDay()));    // update every day
 			}
 			else
 			{
@@ -492,6 +514,17 @@ void MessageWin::initView()
 	grid->addWidget(mOkButton, 0, gridIndex++, AlignHCenter);
 	QWhatsThis::add(mOkButton, i18n("Acknowledge the alarm"));
 
+	if (mShowEdit)
+	{
+		// Edit button
+		mEditButton = new QPushButton(i18n("&Edit..."), topWidget);
+		mEditButton->setFocusPolicy(QWidget::ClickFocus);    // don't allow keyboard selection
+		mEditButton->setFixedSize(mEditButton->sizeHint());
+		connect(mEditButton, SIGNAL(clicked()), SLOT(slotEdit()));
+		grid->addWidget(mEditButton, 0, gridIndex++, AlignHCenter);
+		QWhatsThis::add(mEditButton, i18n("Edit the alarm."));
+	}
+
 	if (!mNoDefer)
 	{
 		// Defer button
@@ -508,7 +541,7 @@ void MessageWin::initView()
 	}
 
 #ifndef WITHOUT_ARTS
-	if (!mAudioFile.isEmpty()  &&  mVolume)
+	if (!mAudioFile.isEmpty()  &&  (mVolume || mFadeVolume > 0))
 	{
 		// Silence button to stop sound repetition
 		QPixmap pixmap = MainBarIcon("player_stop");
@@ -524,15 +557,30 @@ void MessageWin::initView()
 	}
 #endif
 
-	// KAlarm button
 	KIconLoader iconLoader;
+	if (mKMailSerialNumber)
+	{
+		// KMail button
+		QPixmap pixmap = iconLoader.loadIcon(QString::fromLatin1("kmail"), KIcon::MainToolbar);
+		mKMailButton = new QPushButton(topWidget);
+		mKMailButton->setPixmap(pixmap);
+		mKMailButton->setFixedSize(mKMailButton->sizeHint());
+		connect(mKMailButton, SIGNAL(clicked()), SLOT(slotShowKMailMessage()));
+		grid->addWidget(mKMailButton, 0, gridIndex++, AlignHCenter);
+		QToolTip::add(mKMailButton, i18n("Locate this email in KMail", "Locate in KMail"));
+		QWhatsThis::add(mKMailButton, i18n("Locate and highlight this email in KMail"));
+	}
+	else
+		mKMailButton = 0;
+
+	// KAlarm button
 	QPixmap pixmap = iconLoader.loadIcon(QString::fromLatin1(kapp->aboutData()->appName()), KIcon::MainToolbar);
 	mKAlarmButton = new QPushButton(topWidget);
 	mKAlarmButton->setPixmap(pixmap);
 	mKAlarmButton->setFixedSize(mKAlarmButton->sizeHint());
 	connect(mKAlarmButton, SIGNAL(clicked()), SLOT(displayMainWindow()));
 	grid->addWidget(mKAlarmButton, 0, gridIndex++, AlignHCenter);
-	QString actKAlarm = i18n("Activate %1").arg(kapp->aboutData()->programName());
+	QString actKAlarm = i18n("Activate KAlarm");
 	QToolTip::add(mKAlarmButton, actKAlarm);
 	QWhatsThis::add(mKAlarmButton, actKAlarm);
 
@@ -541,13 +589,17 @@ void MessageWin::initView()
 	mOkButton->setEnabled(false);
 	if (mDeferButton)
 		mDeferButton->setEnabled(false);
+	if (mEditButton)
+		mEditButton->setEnabled(false);
+	if (mKMailButton)
+		mKMailButton->setEnabled(false);
 	mKAlarmButton->setEnabled(false);
 
 	topLayout->activate();
 	setMinimumSize(QSize(grid->sizeHint().width() + 2*KDialog::marginHint(), sizeHint().height()));
 
 	WId winid = winId();
-	unsigned long wstate = (Preferences::instance()->modalMessages() ? NET::Modal : 0) | NET::Sticky | NET::StaysOnTop;
+	unsigned long wstate = (Preferences::modalMessages() ? NET::Modal : 0) | NET::Sticky | NET::StaysOnTop;
 	KWin::setState(winid, wstate);
 	KWin::setOnAllDesktops(winid, true);
 }
@@ -560,16 +612,16 @@ void MessageWin::setRemainingTextDay()
 {
 	QString text;
 	int days = QDate::currentDate().daysTo(mDateTime.date());
-	if (days == 0  &&  !mDateTime.isDateOnly())
+	if (days <= 0  &&  !mDateTime.isDateOnly())
 	{
 		// The alarm is due today, so start refreshing every minute
-		DailyTimer::disconnect(this, SLOT(setRemainingTextDay()));
+		MidnightTimer::disconnect(this, SLOT(setRemainingTextDay()));
 		setRemainingTextMinute();
 		MinuteTimer::connect(this, SLOT(setRemainingTextMinute()));   // update every minute
 	}
 	else
 	{
-		if (days == 0)
+		if (days <= 0)
 			text = i18n("Today");
 		else if (days % 7)
 			text = i18n("Tomorrow", "in %n days' time", days);
@@ -588,7 +640,7 @@ void MessageWin::setRemainingTextMinute()
 	QString text;
 	int mins = (QDateTime::currentDateTime().secsTo(mDateTime.dateTime()) + 59) / 60;
 	if (mins < 60)
-		text = i18n("in 1 minute's time", "in %n minutes' time", mins);
+		text = i18n("in 1 minute's time", "in %n minutes' time", (mins > 0 ? mins : 0));
 	else if (mins % 60 == 0)
 		text = i18n("in 1 hour's time", "in %n hours' time", mins/60);
 	else if (mins % 60 == 1)
@@ -619,6 +671,8 @@ void MessageWin::saveProperties(KConfig* config)
 			config->writeEntry(QString::fromLatin1("Time"), mDateTime.dateTime());
 			config->writeEntry(QString::fromLatin1("DateOnly"), mDateTime.isDateOnly());
 		}
+		if (mCloseTime.isValid())
+			config->writeEntry(QString::fromLatin1("Expiry"), mCloseTime);
 #ifndef WITHOUT_ARTS
 		if (mAudioRepeat  &&  mSilenceButton  &&  mSilenceButton->isEnabled())
 		{
@@ -627,8 +681,12 @@ void MessageWin::saveProperties(KConfig* config)
 			config->writeEntry(QString::fromLatin1("Volume"), static_cast<int>(mVolume * 100));
 		}
 #endif
+		config->writeEntry(QString::fromLatin1("Speak"), mSpeak);
 		config->writeEntry(QString::fromLatin1("Height"), height());
+		config->writeEntry(QString::fromLatin1("DeferMins"), mDefaultDeferMinutes);
 		config->writeEntry(QString::fromLatin1("NoDefer"), mNoDefer);
+		config->writeEntry(QString::fromLatin1("NoPostAction"), mNoPostAction);
+		config->writeEntry(QString::fromLatin1("KMailSerial"), mKMailSerialNumber);
 	}
 	else
 		config->writeEntry(QString::fromLatin1("Invalid"), true);
@@ -641,33 +699,56 @@ void MessageWin::saveProperties(KConfig* config)
 */
 void MessageWin::readProperties(KConfig* config)
 {
-	mInvalid       = config->readBoolEntry(QString::fromLatin1("Invalid"), false);
-	mEventID       = config->readEntry(QString::fromLatin1("EventID"));
-	mAlarmType     = KAAlarm::Type(config->readNumEntry(QString::fromLatin1("AlarmType")));
-	mMessage       = config->readEntry(QString::fromLatin1("Message"));
-	mAction        = KAEvent::Action(config->readNumEntry(QString::fromLatin1("Type")));
-	mFont          = config->readFontEntry(QString::fromLatin1("Font"));
-	mBgColour      = config->readColorEntry(QString::fromLatin1("BgColour"));
-	mFgColour      = config->readColorEntry(QString::fromLatin1("FgColour"));
-	mConfirmAck    = config->readBoolEntry(QString::fromLatin1("ConfirmAck"));
+	mInvalid             = config->readBoolEntry(QString::fromLatin1("Invalid"), false);
+	mEventID             = config->readEntry(QString::fromLatin1("EventID"));
+	mAlarmType           = KAAlarm::Type(config->readNumEntry(QString::fromLatin1("AlarmType")));
+	mMessage             = config->readEntry(QString::fromLatin1("Message"));
+	mAction              = KAEvent::Action(config->readNumEntry(QString::fromLatin1("Type")));
+	mFont                = config->readFontEntry(QString::fromLatin1("Font"));
+	mBgColour            = config->readColorEntry(QString::fromLatin1("BgColour"));
+	mFgColour            = config->readColorEntry(QString::fromLatin1("FgColour"));
+	mConfirmAck          = config->readBoolEntry(QString::fromLatin1("ConfirmAck"));
 	QDateTime invalidDateTime;
-	QDateTime dt   = config->readDateTimeEntry(QString::fromLatin1("Time"), &invalidDateTime);
-	bool dateOnly  = config->readBoolEntry(QString::fromLatin1("DateOnly"));
+	QDateTime dt         = config->readDateTimeEntry(QString::fromLatin1("Time"), &invalidDateTime);
+	bool dateOnly        = config->readBoolEntry(QString::fromLatin1("DateOnly"));
 	mDateTime.set(dt, dateOnly);
+	mCloseTime           = config->readDateTimeEntry(QString::fromLatin1("Expiry"), &invalidDateTime);
 #ifndef WITHOUT_ARTS
-	mAudioFile     = config->readPathEntry(QString::fromLatin1("AudioFile"));
-	mVolume        = static_cast<float>(config->readNumEntry(QString::fromLatin1("Volume"))) / 100;
+	mAudioFile           = config->readPathEntry(QString::fromLatin1("AudioFile"));
+	mVolume              = static_cast<float>(config->readNumEntry(QString::fromLatin1("Volume"))) / 100;
+	mFadeVolume          = -1;
+	mFadeSeconds         = 0;
 	if (!mAudioFile.isEmpty())
 		mAudioRepeat = true;
 #endif
-	mRestoreHeight = config->readNumEntry(QString::fromLatin1("Height"));
-	mNoDefer       = config->readBoolEntry(QString::fromLatin1("NoDefer"));
+	mSpeak               = config->readBoolEntry(QString::fromLatin1("Speak"));
+	mRestoreHeight       = config->readNumEntry(QString::fromLatin1("Height"));
+	mDefaultDeferMinutes = config->readNumEntry(QString::fromLatin1("DeferMins"));
+	mNoDefer             = config->readBoolEntry(QString::fromLatin1("NoDefer"));
+	mNoPostAction        = config->readBoolEntry(QString::fromLatin1("NoPostAction"));
+	mKMailSerialNumber   = config->readUnsignedLongNumEntry(QString::fromLatin1("KMailSerial"));
+	mShowEdit            = false;
+	kdDebug(5950) << "MessageWin::readProperties(" << mEventID << ")" << endl;
 	if (mAlarmType != KAAlarm::INVALID_ALARM)
 	{
 		// Recreate the event from the calendar file (if possible)
-		const Event* kcalEvent = mEventID.isNull() ? 0 : AlarmCalendar::activeCalendar()->event(mEventID);
-		if (kcalEvent)
-			mEvent.set(*kcalEvent);
+		if (!mEventID.isEmpty())
+		{
+			const Event* kcalEvent = AlarmCalendar::activeCalendar()->event(mEventID);
+			if (!kcalEvent)
+			{
+				// It's not in the active calendar, so try the displaying calendar
+				AlarmCalendar* cal = AlarmCalendar::displayCalendar();
+				if (cal->isOpen())
+					kcalEvent = cal->event(KAEvent::uid(mEventID, KAEvent::DISPLAYING));
+			}
+			if (kcalEvent)
+			{
+				mEvent.set(*kcalEvent);
+				mEvent.setUid(KAEvent::ACTIVE);    // in case it came from the display calendar
+				mShowEdit = true;
+			}
+		}
 		initView();
 	}
 }
@@ -678,9 +759,12 @@ void MessageWin::readProperties(KConfig* config)
 */
 MessageWin* MessageWin::findEvent(const QString& eventID)
 {
-	for (MessageWin* w = mWindowList.first();  w;  w = mWindowList.next())
+	for (QValueList<MessageWin*>::Iterator it = mWindowList.begin();  it != mWindowList.end();  ++it)
+	{
+		MessageWin* w = *it;
 		if (w->mEventID == eventID  &&  !w->mErrorWindow)
 			return w;
+	}
 	return 0;
 }
 
@@ -697,7 +781,7 @@ void MessageWin::playAudio()
 	}
 	if (!mAudioFile.isEmpty())
 	{
-		if (!mVolume)
+		if (!mVolume  &&  mFadeVolume <= 0)
 			return;    // ensure zero volume doesn't play anything
 #ifdef WITHOUT_ARTS
 		QString play = mAudioFile;
@@ -711,6 +795,48 @@ void MessageWin::playAudio()
 		QTimer::singleShot(0, this, SLOT(slotPlayAudio()));
 #endif
 	}
+	else if (mSpeak)
+	{
+		// The message is to be spoken. In case of error messges,
+		// call it on a timer to allow the window to display first.
+		QTimer::singleShot(0, this, SLOT(slotSpeak()));
+	}
+}
+
+/******************************************************************************
+*  Speak the message.
+*  Called asynchronously to avoid delaying the display of the message.
+*/
+void MessageWin::slotSpeak()
+{
+	DCOPClient* client = kapp->dcopClient();
+	if (!client->isApplicationRegistered("kttsd"))
+	{
+		// kttsd is not running, so start it
+		QString error;
+		if (kapp->startServiceByDesktopName("kttsd", QStringList(), &error))
+		{
+			kdDebug(5950) << "MessageWin::slotSpeak(): failed to start kttsd: " << error << endl;
+			if (!haveErrorMessage(ErrMsg_Speak))
+			{
+				KMessageBox::detailedError(0, i18n("Unable to speak message"), error);
+				clearErrorMessage(ErrMsg_Speak);
+			}
+			return;
+		}
+	}
+	QByteArray  data;
+	QDataStream arg(data, IO_WriteOnly);
+	arg << mMessage << "";
+	if (!client->send("kttsd", "KSpeech", "sayMessage(QString,QString)", data))
+	{
+		kdDebug(5950) << "MessageWin::slotSpeak(): sayMessage() DCOP error" << endl;
+		if (!haveErrorMessage(ErrMsg_Speak))
+		{
+			KMessageBox::detailedError(0, i18n("Unable to speak message"), i18n("DCOP Call sayMessage failed"));
+			clearErrorMessage(ErrMsg_Speak);
+		}
+	}
 }
 
 /******************************************************************************
@@ -727,16 +853,21 @@ void MessageWin::slotPlayAudio()
 	||  !KIO::NetAccess::download(url, mLocalAudioFile, mmw))
 	{
 		kdError(5950) << "MessageWin::playAudio(): Open failure: " << mAudioFile << endl;
-		KMessageBox::error(this, i18n("Cannot open audio file:\n%1").arg(mAudioFile), kapp->aboutData()->programName());
+		if (!haveErrorMessage(ErrMsg_AudioFile))
+		{
+			KMessageBox::error(this, i18n("Cannot open audio file:\n%1").arg(mAudioFile));
+			clearErrorMessage(ErrMsg_AudioFile);
+		}
 		return;
 	}
 	if (!mArtsDispatcher)
 	{
+		mFadeTimer = 0;
 		mPlayTimer = new QTimer(this);
 		connect(mPlayTimer, SIGNAL(timeout()), SLOT(checkAudioPlay()));
 		mArtsDispatcher = new KArtsDispatcher;
 		mPlayedOnce = false;
-		mAudioFileLoadStart = QTime::currentTime();
+		mAudioFileStart = QTime::currentTime();
 		initAudio(true);
 		if (!mPlayObject->object().isNull())
 			checkAudioPlay();
@@ -745,12 +876,13 @@ void MessageWin::slotPlayAudio()
 		{
 			// Output error message now that everything else has been done.
 			// (Outputting it earlier would delay things until it is acknowledged.)
-			bool kmixRunning = kapp->dcopClient()->isApplicationRegistered(KMIX_APP_NAME);
-			QString reason = kmixRunning ? i18n("Error accessing KMix") : i18n("KMix not running");
-			const char* reasonName = kmixRunning ? "KMixError" : "KMixNotRun";
-			KMessageBox::information(this, i18n("Unable to set master volume\n(%1)").arg(reason),
-			                         QString::null, QString::fromLatin1(reasonName));
-			kdWarning(5950) << "Unable to set master volume (KMix " << (kmixRunning ? "DCOP error" : "not running") << ")\n";
+			kdWarning(5950) << "Unable to set master volume (KMix: " << mKMixError << ")\n";
+			if (!haveErrorMessage(ErrMsg_Volume))
+			{
+				KMessageBox::information(this, i18n("Unable to set master volume\n(Error accessing KMix:\n%1)").arg(mKMixError),
+				                         QString::null, QString::fromLatin1("KMixError"));
+				clearErrorMessage(ErrMsg_Volume);
+			}
 		}
 #endif
 	}
@@ -772,26 +904,31 @@ void MessageWin::initAudio(bool firstTime)
 		// Save the existing sound volume setting for restoration afterwards,
 		// and set the desired volume for the alarm.
 		mUsingKMix = false;
-		if (mVolume >= 0)
+		float volume = mVolume;    // initial volume
+		if (volume >= 0)
 		{
+			// The volume has been specified
+			if (mFadeVolume >= 0)
+				volume = mFadeVolume;    // fading, so adjust the initial volume
+
 			// Get the current master volume from KMix
 			int vol = getKMixVolume();
 			if (vol >= 0)
 			{
 				mOldVolume = vol;    // success
 				mUsingKMix = true;
-				setKMixVolume(static_cast<int>(mVolume * 100));
+				setKMixVolume(static_cast<int>(volume * 100));
 			}
 		}
 		if (!mUsingKMix)
 		{
 			/* Adjust within the current master volume, because either
-			 * a) volume is not specified, in which case we want to play at
-			 *    100% of the current master volume setting, or
+			 * a) the volume is not specified, in which case we want to play
+			 *    at 100% of the current master volume setting, or
 			 * b) KMix is not available to set the master volume.
 			 */
 			mOldVolume = sserver.outVolume().scaleFactor();    // save volume for restoration afterwards
-			sserver.outVolume().scaleFactor(mVolume >= 0 ? mVolume : 1);
+			sserver.outVolume().scaleFactor(volume >= 0 ? volume : 1);
 		}
 	}
 	mSilenceButton->setEnabled(true);
@@ -828,9 +965,18 @@ void MessageWin::checkAudioPlay()
 		if (!mPlayedOnce)
 		{
 			// Start playing the file for the first time
-			mAudioFileLoadSecs = mAudioFileLoadStart.secsTo(QTime::currentTime());
+			QTime now = QTime::currentTime();
+			mAudioFileLoadSecs = mAudioFileStart.secsTo(now);
 			if (mAudioFileLoadSecs < 0)
 				mAudioFileLoadSecs += 86400;
+			if (mVolume >= 0  &&  mFadeVolume >= 0  &&  mFadeSeconds > 0)
+			{
+				// Set up volume fade
+				mAudioFileStart = now;
+				mFadeTimer = new QTimer(this);
+				connect(mFadeTimer, SIGNAL(timeout()), SLOT(slotFade()));
+				mFadeTimer->start(1000);     // adjust volume every second
+			}
 			mPlayedOnce = true;
 		}
 		if (mAudioFileLoadSecs < 3)
@@ -885,15 +1031,7 @@ void MessageWin::stopPlay()
 	{
 		// Restore the sound volume to what it was before the sound file
 		// was played, provided that nothing else has modified it since.
-		if (mUsingKMix)
-		{
-			int eventVolume = static_cast<int>(mVolume * 100);
-			int currentVolume = getKMixVolume();
-			// Volume returned isn't always exactly equal to volume set
-			if (currentVolume < 0  ||  abs(currentVolume - eventVolume) < 5)
-				setKMixVolume(static_cast<int>(mOldVolume));
-		}
-		else
+		if (!mUsingKMix)
 		{
 			KArtsServer aserver;
 			Arts::StereoVolumeControl svc = aserver.server().outVolume();
@@ -904,6 +1042,14 @@ void MessageWin::stopPlay()
 			if (currentVolume == eventVolume)
 				svc.scaleFactor(mOldVolume);
 		}
+		else if (mVolume >= 0)
+		{
+			int eventVolume = static_cast<int>(mVolume * 100);
+			int currentVolume = getKMixVolume();
+			// Volume returned isn't always exactly equal to volume set
+			if (currentVolume < 0  ||  abs(currentVolume - eventVolume) < 5)
+				setKMixVolume(static_cast<int>(mOldVolume));
+		}
 	}
 	delete mPlayObject;      mPlayObject = 0;
 	delete mArtsDispatcher;  mArtsDispatcher = 0;
@@ -917,6 +1063,46 @@ void MessageWin::stopPlay()
 #endif
 }
 
+/******************************************************************************
+*  Called every second to fade the volume when the audio file starts playing.
+*/
+void MessageWin::slotFade()
+{
+#ifndef WITHOUT_ARTS
+	QTime now = QTime::currentTime();
+	int elapsed = mAudioFileStart.secsTo(now);
+	if (elapsed < 0)
+		elapsed += 86400;    // it's the next day
+	float volume;
+	if (elapsed >= mFadeSeconds)
+	{
+		// The fade has finished. Set to normal volume.
+		volume = mVolume;
+		delete mFadeTimer;
+		mFadeTimer = 0;
+		if (!mVolume)
+		{
+			kdDebug(5950) << "MessageWin::slotFade(0)\n";
+			stopPlay();
+			return;
+		}
+	}
+	else
+		volume = mFadeVolume  +  ((mVolume - mFadeVolume) * elapsed) / mFadeSeconds;
+	kdDebug(5950) << "MessageWin::slotFade(" << volume << ")\n";
+	if (mArtsDispatcher)
+	{
+		if (mUsingKMix)
+			setKMixVolume(static_cast<int>(volume * 100));
+		else if (mArtsDispatcher)
+		{
+			KArtsServer aserver;
+			aserver.server().outVolume().scaleFactor(volume);
+		}
+	}
+#endif
+}
+
 #ifndef WITHOUT_ARTS
 /******************************************************************************
 *  Get the master volume from KMix.
@@ -924,7 +1110,7 @@ void MessageWin::stopPlay()
 */
 int MessageWin::getKMixVolume()
 {
-	if (!runKMix())     // start KMix if it isn't already running
+	if (!KAlarm::runProgram(KMIX_APP_NAME, KMIX_DCOP_WINDOW, mKMixName, mKMixError))   // start KMix if it isn't already running
 		return -1;
 	QByteArray  data, replyData;
 	QCString    replyType;
@@ -945,7 +1131,7 @@ void MessageWin::setKMixVolume(int percent)
 {
 	if (!mUsingKMix)
 		return;
-	if (!runKMix())     // start KMix if it isn't already running
+	if (!KAlarm::runProgram(KMIX_APP_NAME, KMIX_DCOP_WINDOW, mKMixName, mKMixError))   // start KMix if it isn't already running
 		return;
 	QByteArray  data;
 	QDataStream arg(data, IO_WriteOnly);
@@ -953,27 +1139,6 @@ void MessageWin::setKMixVolume(int percent)
 	if (!kapp->dcopClient()->send(mKMixName, KMIX_DCOP_OBJECT, "setMasterVolume(int)", data))
 		kdError(5950) << "MessageWin::setKMixVolume(): kmix dcop call failed\n";
 }
-
-/******************************************************************************
-*  Start KMix if it isn't already running.
-*  Reply = true if KMix is now running.
-*/
-bool MessageWin::runKMix()
-{
-	if (mKMixName.isEmpty())
-		mKMixName = KMIX_APP_NAME;
-	if (!kapp->dcopClient()->isApplicationRegistered(mKMixName))
-	{
-		// KMix is not already running, so start it
-		if (KApplication::startServiceByDesktopName(QString::fromLatin1(KMIX_APP_NAME), QString::null, &mKMixError, &mKMixName))
-			return false;
-		// Minimise its window - don't use hide() since this would remove all
-		// trace of it from the panel if it is not configured to be docked in
-		// the system tray.
-		kapp->dcopClient()->send(mKMixName, KMIX_DCOP_WINDOW, "minimize()", QString::null);
-	}
-	return true;
-}
 #endif
 
 /******************************************************************************
@@ -993,7 +1158,7 @@ void MessageWin::repeat(const KAAlarm& alarm)
 	if (kcalEvent)
 	{
 		mAlarmType = alarm.type();    // store new alarm type for use if it is later deferred
-		if (!mDeferDlg  ||  Preferences::instance()->modalMessages())
+		if (!mDeferDlg  ||  Preferences::modalMessages())
 		{
 			raise();
 			playAudio();
@@ -1013,7 +1178,17 @@ void MessageWin::repeat(const KAAlarm& alarm)
 */
 void MessageWin::show()
 {
-	if (Preferences::instance()->messageButtonDelay() == 0)
+	if (mCloseTime.isValid())
+	{
+		// Set a timer to auto-close the window
+		int delay = QDateTime::currentDateTime().secsTo(mCloseTime);
+		if (delay < 0)
+			delay = 0;
+		QTimer::singleShot(delay * 1000, this, SLOT(close()));
+		if (!delay)
+			return;    // don't show the window if auto-closing is already due
+	}
+	if (Preferences::messageButtonDelay() == 0)
 		move(0, 0);
 	MainWindowBase::show();
 }
@@ -1061,7 +1236,7 @@ void MessageWin::showEvent(QShowEvent* se)
 				KAlarm::readConfigWindowSize("FileMessage", s);
 			resize(s);
 
-			mButtonDelay = Preferences::instance()->messageButtonDelay() * 1000;
+			mButtonDelay = Preferences::messageButtonDelay() * 1000;
 			if (!mButtonDelay)
 			{
 				/* Try to ensure that the window can't accidentally be acknowledged
@@ -1075,10 +1250,20 @@ void MessageWin::showEvent(QShowEvent* se)
 				 *      See the Qt documentation on window geometry for more details.
 				 */
 				// PROBLEM: The frame size is not known yet!
+
+				/* Find the usable area of the desktop or, if the desktop comprises
+				 * multiple screens, the usable area of the current screen. (If the
+				 * message is displayed on a screen other than that currently being
+				 * worked with, it might not be noticed.)
+				 */
+				QPoint cursor = QCursor::pos();
 				if (!mWinModule)
 					mWinModule = new KWinModule(0, KWinModule::INFO_DESKTOP);
 				QRect desk = mWinModule->workArea();
-				QPoint cursor = QCursor::pos();
+				QDesktopWidget* dw = QApplication::desktop();
+				if (dw->numScreens() > 1)
+					desk &= dw->screenGeometry(dw->screenNumber(cursor));
+
 				QRect frame = frameGeometry();
 				QRect rect  = geometry();
 				// Find the offsets from the outside of the frame to the edges of the OK button
@@ -1170,8 +1355,12 @@ void MessageWin::enableButtons()
 {
 	mOkButton->setEnabled(true);
 	mKAlarmButton->setEnabled(true);
-	if (mDeferButton)
+	if (mDeferButton  &&  !mDisableDeferral)
 		mDeferButton->setEnabled(true);
+	if (mEditButton)
+		mEditButton->setEnabled(true);
+	if (mKMailButton)
+		mKMailButton->setEnabled(true);
 }
 
 /******************************************************************************
@@ -1205,25 +1394,99 @@ void MessageWin::resizeEvent(QResizeEvent* re)
 */
 void MessageWin::closeEvent(QCloseEvent* ce)
 {
-	if (mConfirmAck  &&  !mDeferClosing  &&  !theApp()->sessionClosingDown())
+	// Don't prompt or delete the alarm from the display calendar if the session is closing
+	if (!theApp()->sessionClosingDown())
 	{
-		// Ask for confirmation of acknowledgement. Use warningYesNo() because its default is No.
-		if (KMessageBox::warningYesNo(this, i18n("Do you really want to acknowledge this alarm?"),
-		                                    i18n("Acknowledge Alarm"), i18n("&Acknowledge"), KStdGuiItem::cancel())
-		    != KMessageBox::Yes)
+		if (mConfirmAck  &&  !mNoCloseConfirm)
 		{
-			ce->ignore();
-			return;
+			// Ask for confirmation of acknowledgement. Use warningYesNo() because its default is No.
+			if (KMessageBox::warningYesNo(this, i18n("Do you really want to acknowledge this alarm?"),
+			                                    i18n("Acknowledge Alarm"), i18n("&Acknowledge"), KStdGuiItem::cancel())
+			    != KMessageBox::Yes)
+			{
+				ce->ignore();
+				return;
+			}
+		}
+		if (!mEventID.isNull())
+		{
+			// Delete from the display calendar
+			KAlarm::deleteDisplayEvent(KAEvent::uid(mEventID, KAEvent::DISPLAYING));
 		}
 	}
-	if (!mEventID.isNull())
-	{
-		// Delete from the display calendar
-		KAlarm::deleteDisplayEvent(KAEvent::uid(mEventID, KAEvent::DISPLAYING));
-	}
 	MainWindowBase::closeEvent(ce);
 }
 
+/******************************************************************************
+*  Called when the KMail button is clicked.
+*  Tells KMail to display the email message displayed in this message window.
+*/
+void MessageWin::slotShowKMailMessage()
+{
+	kdDebug(5950) << "MessageWin::slotShowKMailMessage()\n";
+	if (!mKMailSerialNumber)
+		return;
+	QString err = KAlarm::runKMail(false);
+	if (!err.isNull())
+	{
+		KMessageBox::sorry(this, err);
+		return;
+	}
+	QCString    replyType;
+	QByteArray  data, replyData;
+	QDataStream arg(data, IO_WriteOnly);
+	arg << (Q_UINT32)mKMailSerialNumber << QString::null;
+	if (kapp->dcopClient()->call("kmail", KMAIL_DCOP_OBJECT, "showMail(Q_UINT32,QString)", data, replyType, replyData)
+	&&  replyType == "bool")
+	{
+		bool result;
+		QDataStream replyStream(replyData, IO_ReadOnly);
+		replyStream >> result;
+		if (result)
+			return;    // success
+	}
+	kdError(5950) << "MessageWin::slotShowKMailMessage(): kmail dcop call failed\n";
+	KMessageBox::sorry(this, i18n("Unable to locate this email in KMail"));
+}
+
+/******************************************************************************
+*  Called when the Edit... button is clicked.
+*  Displays the alarm edit dialog.
+*/
+void MessageWin::slotEdit()
+{
+	kdDebug(5950) << "MessageWin::slotEdit()" << endl;
+	EditAlarmDlg editDlg(false, i18n("Edit Alarm"), this, "editDlg", &mEvent);
+	if (editDlg.exec() == QDialog::Accepted)
+	{
+		KAEvent event;
+		editDlg.getEvent(event);
+
+		// Update the displayed lists and the calendar file
+		KAlarm::UpdateStatus status;
+		if (AlarmCalendar::activeCalendar()->event(mEventID))
+		{
+			// The old alarm hasn't expired yet, so replace it
+			status = KAlarm::modifyEvent(mEvent, event, 0, &editDlg);
+			Undo::saveEdit(mEvent, event);
+		}
+		else
+		{
+			// The old event has expired, so simply create a new one
+			status = KAlarm::addEvent(event, 0, &editDlg);
+			Undo::saveAdd(event);
+		}
+
+		if (status == KAlarm::UPDATE_KORG_ERR)
+			KAlarm::displayKOrgUpdateError(&editDlg, KAlarm::KORG_ERR_MODIFY, 1);
+		KAlarm::outputAlarmWarnings(&editDlg, &event);
+
+		// Close the alarm window
+		mNoCloseConfirm = true;   // allow window to close without confirmation prompt
+		close();
+	}
+}
+
 /******************************************************************************
 * Set up to disable the defer button when the deferral limit is reached.
 */
@@ -1231,9 +1494,9 @@ void MessageWin::setDeferralLimit(const KAEvent& event)
 {
 	if (mDeferButton)
 	{
-		// Ensure that defer button is disabled when the deferral limit is reached
 		mDeferLimit = event.deferralLimit().dateTime();
-		DailyTimer::connect(this, SLOT(checkDeferralLimit()));   // check every day
+		MidnightTimer::connect(this, SLOT(checkDeferralLimit()));   // check every day
+		mDisableDeferral = false;
 		checkDeferralLimit();
 	}
 }
@@ -1254,7 +1517,7 @@ void MessageWin::checkDeferralLimit()
 	int n = QDate::currentDate().daysTo(mDeferLimit.date());
 	if (n > 0)
 		return;
-	DailyTimer::disconnect(this, SLOT(checkDeferralLimit()));
+	MidnightTimer::disconnect(this, SLOT(checkDeferralLimit()));
 	if (n == 0)
 	{
 		// The deferral limit will be reached today
@@ -1266,6 +1529,7 @@ void MessageWin::checkDeferralLimit()
 		}
 	}
 	mDeferButton->setEnabled(false);
+	mDisableDeferral = true;
 }
 
 /******************************************************************************
@@ -1276,19 +1540,25 @@ void MessageWin::slotDefer()
 {
 	mDeferDlg = new DeferAlarmDlg(i18n("Defer Alarm"), QDateTime::currentDateTime().addSecs(60),
 	                              false, this, "deferDlg");
+	if (mDefaultDeferMinutes > 0)
+		mDeferDlg->setDeferMinutes(mDefaultDeferMinutes);
 	mDeferDlg->setLimit(mEventID);
-	if (!Preferences::instance()->modalMessages())
+	if (!Preferences::modalMessages())
 		lower();
 	if (mDeferDlg->exec() == QDialog::Accepted)
 	{
-		DateTime dateTime = mDeferDlg->getDateTime();
+		DateTime dateTime  = mDeferDlg->getDateTime();
+		int      delayMins = mDeferDlg->deferMinutes();
 		const Event* kcalEvent = mEventID.isNull() ? 0 : AlarmCalendar::activeCalendar()->event(mEventID);
 		if (kcalEvent)
 		{
 			// The event still exists in the calendar file.
 			KAEvent event(*kcalEvent);
 			bool repeat = event.defer(dateTime, (mAlarmType & KAAlarm::REMINDER_ALARM), true);
-			KAlarm::updateEvent(event, 0, true, !repeat);
+			event.setDeferDefaultMinutes(delayMins);
+			KAlarm::updateEvent(event, 0, mDeferDlg, true, !repeat);
+			if (event.deferred())
+				mNoPostAction = true;
 		}
 		else
 		{
@@ -1303,12 +1573,16 @@ void MessageWin::slotDefer()
 			{
 				// The event doesn't exist any more !?!, so create a new one
 				event.set(dateTime.dateTime(), mMessage, mBgColour, mFgColour, mFont, mAction, mLateCancel, mFlags);
-				event.setAudioFile(mAudioFile, mVolume);
+				event.setAudioFile(mAudioFile, mVolume, mFadeVolume, mFadeSeconds);
 				event.setArchive();
 				event.setEventID(mEventID);
 			}
+			event.setDeferDefaultMinutes(delayMins);
 			// Add the event back into the calendar file, retaining its ID
-			KAlarm::addEvent(event, 0, true);
+			// and not updating KOrganizer
+			KAlarm::addEvent(event, 0, mDeferDlg, true, false);
+			if (event.deferred())
+				mNoPostAction = true;
 			if (kcalEvent)
 			{
 				event.setUid(KAEvent::EXPIRED);
@@ -1321,7 +1595,7 @@ void MessageWin::slotDefer()
 			// so start it if necessary so that the deferred alarm will be shown.
 			theApp()->displayTrayIcon(true);
 		}
-		mDeferClosing = true;   // allow window to close without confirmation prompt
+		mNoCloseConfirm = true;   // allow window to close without confirmation prompt
 		close();
 	}
 	else
@@ -1339,6 +1613,31 @@ void MessageWin::displayMainWindow()
 	KAlarm::displayMainWindowSelected(mEventID);
 }
 
+/******************************************************************************
+* Check whether the specified error message is already displayed for this
+* alarm, and note that it will now be displayed.
+* Reply = true if message is already displayed.
+*/
+bool MessageWin::haveErrorMessage(unsigned msg) const
+{
+	if (!mErrorMessages.contains(mEventID))
+		mErrorMessages.insert(mEventID, 0);
+	bool result = (mErrorMessages[mEventID] & msg);
+	mErrorMessages[mEventID] |= msg;
+	return result;
+}
+
+void MessageWin::clearErrorMessage(unsigned msg) const
+{
+	if (mErrorMessages.contains(mEventID))
+	{
+		if (mErrorMessages[mEventID] == msg)
+			mErrorMessages.remove(mEventID);
+		else
+			mErrorMessages[mEventID] &= ~msg;
+	}
+}
+
 
 /*=============================================================================
 = Class MWMimeSourceFactory
