@@ -18,6 +18,8 @@
 #include <qfile.h>
 #include <qtextstream.h>
 #include <qmultilineedit.h>
+#include <qpopupmenu.h>
+#include "mainwindow.h"
 
 karmPart::karmPart( QWidget *parentWidget, const char *widgetName,
                                   QObject *parent, const char *name )
@@ -419,6 +421,14 @@ extern "C"
     }
 }
 
+void karmPart::contextMenuRequest( QListViewItem*, const QPoint& point, int )
+{
+    QPopupMenu* pop = dynamic_cast<QPopupMenu*>(
+                          factory()->container( i18n( "task_popup" ), this ) );
+    if ( pop )
+      pop->popup( point );
+}
+
 //----------------------------------------------------------------------------
 //
 //                       D C O P   I N T E R F A C E
@@ -492,7 +502,7 @@ int karmPart::addTask( const QString& taskname )
 
 QString karmPart::setPerCentComplete( const QString& taskName, int perCent )
 {
-  int index;
+  int index = 0;
   QString err="no such task";
   for (int i=0; i<_taskView->count(); i++)
   {
@@ -683,7 +693,11 @@ QString karmPart::importplannerfile( QString fileName )
   return _taskView->importPlanner(fileName);
 }
 
-
+void karmPart::startNewSession()
+{
+  _taskView->startNewSession();
+  _taskView->save();
+} 
 
 #include <qpopupmenu.h>
 #include "karm_part.moc"
