@@ -15,8 +15,8 @@
  *   You should have received a copy of the GNU General Public License along
  *   with this program; if not, write to the
  *      Free Software Foundation, Inc.
- *      59 Temple Place - Suite 330
- *      Boston, MA  02111-1307  USA.
+ *      51 Franklin Street, Fifth Floor
+ *      Boston, MA  02110-1301  USA.
  *
  */
 
@@ -65,7 +65,7 @@
 
 using namespace std;
 
-KarmStorage *KarmStorage::_instance = 0;    
+KarmStorage *KarmStorage::_instance = 0;
 static long linenr;  // how many lines written by printTaskHistory so far
 
 
@@ -80,7 +80,9 @@ KarmStorage::KarmStorage()
   _calendar = 0;
 }
 
-QString KarmStorage::load (TaskView* view, const Preferences* preferences)
+QString KarmStorage::load (TaskView* view, const Preferences* preferences, QString fileName )
+// loads data from filename into view. If no filename is given, filename from preferences is used.
+// filename might be of use if this program is run as embedded konqueror plugin.
 {
   // When I tried raising an exception from this method, the compiler
   // complained that exceptions are not allowed.  Not sure how apps
@@ -92,9 +94,10 @@ QString KarmStorage::load (TaskView* view, const Preferences* preferences)
 
   QString err;
   KEMailSettings settings;
+  if ( fileName.isEmpty() ) fileName = preferences->iCalFile();
 
   // If same file, don't reload
-  if ( preferences->iCalFile() == _icalfile ) return err;
+  if ( fileName == _icalfile ) return err;
 
 
   // If file doesn't exist, create a blank one to avoid ResourceLocal load
@@ -104,48 +107,44 @@ QString KarmStorage::load (TaskView* view, const Preferences* preferences)
   {
     int handle;
     handle = open (
-        QFile::encodeName( preferences->iCalFile() ),
+        QFile::encodeName( fileName ),
         O_CREAT|O_EXCL|O_WRONLY,
         S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH
         );
     if (handle != -1) close(handle);
   }
 
-  if ( _calendar) closeStorage(view);
-  else _calendar = new KCal::CalendarResources();
+  if ( _calendar)
+    closeStorage(view);
 
   // Create local file resource and add to resources
-  _icalfile = preferences->iCalFile();
+  _icalfile = fileName;
 
-  KCal::ResourceCalendar *l;
+  KCal::ResourceCached *resource;
   if ( remoteResource( _icalfile ) )
   {
-    KURL download = KURL( _icalfile );
-    KURL upload   = KURL( _icalfile );
-    l = new KCal::ResourceRemote( upload, download );
+    KURL url( _icalfile );
+    resource = new KCal::ResourceRemote( url, url ); // same url for upload and download
   }
   else
   {
-    l = new KCal::ResourceLocal( _icalfile );
+    resource = new KCal::ResourceLocal( _icalfile );
   }
+  _calendar = resource;
 
-  QObject::connect (l, SIGNAL(resourceChanged(ResourceCalendar *)),
+  QObject::connect (_calendar, SIGNAL(resourceChanged(ResourceCalendar *)),
   	            view, SLOT(iCalFileModified(ResourceCalendar *)));
-  l->setTimeZoneId( KPimPrefs::timezone() );
-  l->setResourceName( QString::fromLatin1("KArm") );
-  l->open();
-  l->load();
-
-  KCal::CalendarResourceManager *m = _calendar->resourceManager();
-  m->add(l);
-  m->setStandardResource(l);
+  _calendar->setTimeZoneId( KPimPrefs::timezone() );
+  _calendar->setResourceName( QString::fromLatin1("KArm") );
+  _calendar->open();
+  _calendar->load();
 
   // Claim ownership of iCalendar file if no one else has.
-  KCal::Person owner = _calendar->getOwner();
+  KCal::Person owner = resource->getOwner();
   if ( owner.isEmpty() )
   {
-    _calendar->setOwner( KCal::Person( 
-          settings.getSetting( KEMailSettings::RealName ), 
+    resource->setOwner( KCal::Person(
+          settings.getSetting( KEMailSettings::RealName ),
           settings.getSetting( KEMailSettings::EmailAddress ) ) );
   }
 
@@ -184,13 +183,7 @@ QString KarmStorage::load (TaskView* view, const Preferences* preferences)
       Task* task = new Task(*todo, view);
       map.insert( (*todo)->uid(), task );
       view->setRootIsDecorated(true);
-      if ((*todo)->isCompleted())
-      {
-        task->setEnabled(false);
-        task->setOpen(false);
-      }
-      else
-        task->setOpen(true);
+      task->setPixmapProgress();
     }
 
     // Load each task under it's parent task.
@@ -219,7 +212,13 @@ QString KarmStorage::load (TaskView* view, const Preferences* preferences)
 
   return err;
 }
- 
+
+QString KarmStorage::icalfile()
+{
+  kdDebug(5970) << "Entering KarmStorage::icalfile" << endl;
+  return _icalfile;
+}
+
 QString KarmStorage::buildTaskView(KCal::ResourceCalendar *rc, TaskView *view)
 // makes *view contain the tasks out of *rc.
 {
@@ -227,10 +226,23 @@ QString KarmStorage::buildTaskView(KCal::ResourceCalendar *rc, TaskView *view)
   KCal::Todo::List todoList;
   KCal::Todo::List::ConstIterator todo;
   QDict< Task > map;
-  
-  // 1. delete old tasks
+  vector<QString> runningTasks;
+  vector<QDateTime> startTimes;
+
+  // remember tasks that are running and their start times
+  for ( int i=0; i<view->count(); i++)
+  {
+    if ( view->item_at_index(i)->isRunning() )
+    {
+      runningTasks.push_back( view->item_at_index(i)->uid() );
+      startTimes.push_back( view->item_at_index(i)->lastStart() );
+    }
+  }
+
+  //view->stopAllTimers();
+  // delete old tasks
   while (view->item_at_index(0)) view->item_at_index(0)->cut();
-  
+
   // 1. insert tasks form rc into taskview
   // 1.1. Build dictionary to look up Task object from Todo uid.  Each task is a
   // QListViewItem, and is initially added with the view as the parent.
@@ -240,13 +252,7 @@ QString KarmStorage::buildTaskView(KCal::ResourceCalendar *rc, TaskView *view)
     Task* task = new Task(*todo, view);
     map.insert( (*todo)->uid(), task );
     view->setRootIsDecorated(true);
-    if ((*todo)->isCompleted())
-    {
-      task->setEnabled(false);
-      task->setOpen(false);
-    }
-    else
-      task->setOpen(true);
+    task->setPixmapProgress();
   }
 
   // 1.1. Load each task under it's parent task.
@@ -268,6 +274,22 @@ QString KarmStorage::buildTaskView(KCal::ResourceCalendar *rc, TaskView *view)
       if (!err) task->move( newParent);
     }
   }
+
+  view->clearActiveTasks();
+  // restart tasks that have been running with their start times
+  for ( int i=0; i<view->count(); i++)
+  {
+    for ( unsigned int n=0; n<runningTasks.size(); n++)
+    {
+      if ( runningTasks[n] == view->item_at_index(i)->uid() )
+      {
+        view->startTimerFor( view->item_at_index(i), startTimes[n] ); 
+      }
+    }
+  }
+  
+  view->refresh();
+
   return err;
 }
 
@@ -275,11 +297,9 @@ void KarmStorage::closeStorage(TaskView* view)
 {
   if ( _calendar )
   {
-
     _calendar->close();
-
-    KCal::CalendarResourceManager *m = _calendar->resourceManager();
-    m->remove( m->standardResource() );
+    delete _calendar;
+    _calendar = 0;
 
     view->clear();
   }
@@ -287,46 +307,59 @@ void KarmStorage::closeStorage(TaskView* view)
 
 QString KarmStorage::save(TaskView* taskview)
 {
-  QString err="";
+  kdDebug(5970) << "entering KarmStorage::save" << endl;
+  QString err=QString();
 
   QPtrStack< KCal::Todo > parents;
 
   for (Task* task=taskview->first_child(); task; task = task->nextSibling())
   {
-    writeTaskAsTodo(task, 1, parents );
+    err=writeTaskAsTodo(task, 1, parents );
   }
 
-  if ( !_calendar->save( 
-        _calendar->requestSaveTicket( _calendar->resourceManager()->standardResource() )
-        )
-      ) 
+  if ( !saveCalendar() )
   {
     err="Could not save";
   }
 
-  kdDebug(5970)
-    << "KarmStorage::save : wrote "
-    << taskview->count() << " tasks to " << _icalfile << endl;
+  if ( err.isEmpty() )
+  {
+    kdDebug(5970)
+      << "KarmStorage::save : wrote "
+      << taskview->count() << " tasks to " << _icalfile << endl;
+  }
+  else
+  {
+    kdWarning(5970) << "KarmStorage::save : " << err << endl;
+  }
+
   return err;
 }
 
-void KarmStorage::writeTaskAsTodo(Task* task, const int level,
+QString KarmStorage::writeTaskAsTodo(Task* task, const int level,
     QPtrStack< KCal::Todo >& parents )
 {
+  QString err;
   KCal::Todo* todo;
 
   todo = _calendar->todo(task->uid());
+  if ( !todo )
+  {
+    kdDebug(5970) << "Could not get todo from calendar" << endl;
+    return "Could not get todo from calendar";
+  }
   task->asTodo(todo);
   if ( !parents.isEmpty() ) todo->setRelatedTo( parents.top() );
   parents.push( todo );
 
-  for (Task* nextTask = task->firstChild(); nextTask;
-      nextTask = nextTask->nextSibling() )
+  for ( Task* nextTask = task->firstChild(); nextTask;
+        nextTask = nextTask->nextSibling() )
   {
-    writeTaskAsTodo(nextTask, level+1, parents );
+    err = writeTaskAsTodo(nextTask, level+1, parents );
   }
 
   parents.pop();
+  return err;
 }
 
 bool KarmStorage::isEmpty()
@@ -539,17 +572,17 @@ void KarmStorage::adjustFromLegacyFileFormat(Task* task)
 //----------------------------------------------------------------------------
 // Routines that handle Comma-Separated Values export file format.
 //
-QString KarmStorage::exportcsvFile( TaskView *taskview, 
+QString KarmStorage::exportcsvFile( TaskView *taskview,
                                     const ReportCriteria &rc )
 {
   QString delim = rc.delimiter;
   QString dquote = rc.quote;
   QString double_dquote = dquote + dquote;
   bool to_quote = true;
-  
+
   QString err;
   Task* task;
-  int maxdepth=0; 
+  int maxdepth=0;
 
   kdDebug(5970)
     << "KarmStorage::exportcsvFile: " << rc.url << endl;
@@ -565,21 +598,21 @@ QString KarmStorage::exportcsvFile( TaskView *taskview,
   QSize dialogsize;
   dialogsize.setWidth(width);
   dialog.setInitialSize( dialogsize, true );
-   
+
   if ( taskview->count() > 1 ) dialog.show();
-    
+
   QString retval;
 
   // Find max task depth
   int tasknr = 0;
   while ( tasknr < taskview->count() && !dialog.wasCancelled() )
-  { 
+  {
     dialog.progressBar()->advance( 1 );
-    if ( tasknr % 15 == 0 ) kapp->processEvents(); // repainting is slow 
-    if ( taskview->item_at_index(tasknr)->depth() > maxdepth ) 
-      maxdepth = taskview->item_at_index(tasknr)->depth(); 
+    if ( tasknr % 15 == 0 ) kapp->processEvents(); // repainting is slow
+    if ( taskview->item_at_index(tasknr)->depth() > maxdepth )
+      maxdepth = taskview->item_at_index(tasknr)->depth();
     tasknr++;
-  } 
+  }
 
   // Export to file
   tasknr = 0;
@@ -591,30 +624,30 @@ QString KarmStorage::exportcsvFile( TaskView *taskview,
 
     // indent the task in the csv-file:
     for ( int i=0; i < task->depth(); ++i ) retval += delim;
-      
+
     /*
     // CSV compliance
-    // Surround the field with quotes if the field contains 
+    // Surround the field with quotes if the field contains
     // a comma (delim) or a double quote
     if (task->name().contains(delim) || task->name().contains(dquote))
-      to_quote = TRUE;
+      to_quote = true;
     else
-      to_quote = FALSE;
+      to_quote = false;
     */
     to_quote = true;
-      
+
     if (to_quote)
       retval += dquote;
-        
-    // Double quotes replaced by a pair of consecutive double quotes 
+
+    // Double quotes replaced by a pair of consecutive double quotes
     retval += task->name().replace( dquote, double_dquote );
-      
+
     if (to_quote)
       retval += dquote;
-      
+
     // maybe other tasks are more indented, so to align the columns:
     for ( int i = 0; i < maxdepth - task->depth(); ++i ) retval += delim;
-      
+
     retval += delim + formatTime( task->sessionTime(),
                                    rc.decimalMinutes )
            + delim + formatTime( task->time(),
@@ -626,10 +659,10 @@ QString KarmStorage::exportcsvFile( TaskView *taskview,
            + "\n";
     tasknr++;
   }
-  
+
   // save, either locally or remote
   if ((rc.url.isLocalFile()) || (!rc.url.url().contains("/")))
-  {    
+  {
     QString filename=rc.url.path();
     if (filename.isEmpty()) filename=rc.url.url();
     QFile f( filename );
@@ -647,16 +680,16 @@ QString KarmStorage::exportcsvFile( TaskView *taskview,
   else // use remote file
   {
     KTempFile tmpFile;
-    if ( tmpFile.status() != 0 ) err = QString::fromLatin1( "Unable to get temporary file" ); 
+    if ( tmpFile.status() != 0 ) err = QString::fromLatin1( "Unable to get temporary file" );
     else
     {
       QTextStream *stream=tmpFile.textStream();
       *stream << retval;
-      tmpFile.close(); 
+      tmpFile.close();
       if (!KIO::NetAccess::upload( tmpFile.name(), rc.url, 0 )) err=QString::fromLatin1("Could not upload");
     }
   }
-  
+
   return err;
 }
 
@@ -681,6 +714,12 @@ QString KarmStorage::addTask(const Task* task, const Task* parent)
       todo->setRelatedTo(_calendar->todo(parent->uid()));
     uid = todo->uid();
   }
+  else
+  {
+    // Most likely a lock could not be pulled, although there are other
+    // possiblities (like a really confused resource manager).
+    uid = "";
+  }
 
   return uid;
 }
@@ -711,8 +750,7 @@ bool KarmStorage::removeTask(Task* task)
   _calendar->deleteTodo(todo);
 
   // Save entire file
-  _calendar->save(_calendar->requestSaveTicket
-    (_calendar->resourceManager()->standardResource()));
+  saveCalendar();
 
   return true;
 }
@@ -728,27 +766,26 @@ void KarmStorage::addComment(const Task* task, const QString& comment)
   // transition to using the addComment method, we need this second param.
   QString s = comment;
 
-  // TODO: Use libkcal comments 
+  // TODO: Use libkcal comments
   // todo->addComment(comment);
   // temporary
   todo->setDescription(task->comment());
 
-  _calendar->save(_calendar->requestSaveTicket
-    ( _calendar->resourceManager()->standardResource() ));
+  saveCalendar();
 }
 
 long KarmStorage::printTaskHistory (
-        const Task               *task, 
-        const QMap<QString,long> &taskdaytotals, 
-        QMap<QString,long>       &daytotals, 
+        const Task               *task,
+        const QMap<QString,long> &taskdaytotals,
+        QMap<QString,long>       &daytotals,
         const QDate              &from,
-        const QDate              &to, 
+        const QDate              &to,
         const int                level,
 	vector <QString>         &matrix,
         const ReportCriteria     &rc)
 // to>=from is precondition
 {
-  long ownline=linenr++; // the how many-th instance of this function is this 
+  long ownline=linenr++; // the how many-th instance of this function is this
   long colrectot=0;      // colum where to write the task's total recursive time
   vector <QString> cell; // each line of the matrix is stored in an array of cells, one containing the recursive total
   long add;              // total recursive time of all subtasks
@@ -791,29 +828,29 @@ long KarmStorage::printTaskHistory (
     day = day.addDays(1);
   }
 
-  // Total for task 
+  // Total for task
   cell.push_back(QString::fromLatin1("%1").arg(formatTime(sum/60, rc.decimalMinutes)));
-  
+
   // room for the recursive total time (that cannot be calculated now)
   cell.push_back(delim);
   colrectot = cell.size();
   cell.push_back("???");
   cell.push_back(delim);
-  
+
   // Task name
   for ( int i = level + 1; i > 0; i-- ) cell.push_back(delim);
 
   /*
   // CSV compliance
-  // Surround the field with quotes if the field contains 
+  // Surround the field with quotes if the field contains
   // a comma (delim) or a double quote
   to_quote = task->name().contains(delim) || task->name().contains(dquote);
   */
-  to_quote = true; 
+  to_quote = true;
   if ( to_quote) cell.push_back(dquote);
 
 
-  // Double quotes replaced by a pair of consecutive double quotes 
+  // Double quotes replaced by a pair of consecutive double quotes
   cell.push_back(task->name().replace( dquote, double_dquote ));
 
   if ( to_quote) cell.push_back(dquote);
@@ -840,28 +877,28 @@ QString KarmStorage::report( TaskView *taskview, const ReportCriteria &rc )
       err = exportcsvHistory( taskview, rc.from, rc.to, rc );
   else if ( rc.reportType == ReportCriteria::CSVTotalsExport )
       err = exportcsvFile( taskview, rc );
-  else
+  else {
       // hmmmm ... assert(0)?
-      ;
+  }
   return err;
 }
 
 // export history report as csv, all tasks X all dates in one block
-QString KarmStorage::exportcsvHistory ( TaskView      *taskview, 
-                                            const QDate   &from, 
+QString KarmStorage::exportcsvHistory ( TaskView      *taskview,
+                                            const QDate   &from,
                                             const QDate   &to,
                                             const ReportCriteria &rc)
 {
   QString delim = rc.delimiter;
   const QString cr = QString::fromLatin1("\n");
   QString err;
-  
+
   // below taken from timekard.cpp
   QString retval;
   QString taskhdr, totalhdr;
   QString line, buf;
   long sum;
-  
+
   QValueList<HistoryEvent> events;
   QValueList<HistoryEvent>::iterator event;
   QMap<QString, long> taskdaytotals;
@@ -871,12 +908,12 @@ QString KarmStorage::exportcsvHistory ( TaskView      *taskview,
   QDate dayheading;
 
   // parameter-plausi
-  if ( from > to ) 
+  if ( from > to )
   {
     err = QString::fromLatin1 (
             "'to' has to be a date later than or equal to 'from'.");
   }
- 
+
   // header
   retval += i18n("Task History\n");
   retval += i18n("From %1 to %2")
@@ -891,7 +928,7 @@ QString KarmStorage::exportcsvHistory ( TaskView      *taskview,
   events = taskview->getHistory(from, to);
   taskdaytotals.clear();
   daytotals.clear();
- 
+
   // Build lookup dictionary used to output data in table cells.  keys are
   // in this format: YYYYMMDD_NNNNNN, where Y = year, M = month, d = day and
   // NNNNN = the VTODO uid.  The value is the total seconds logged against
@@ -903,14 +940,14 @@ QString KarmStorage::exportcsvHistory ( TaskView      *taskview,
     daytaskkey = QString(QString::fromLatin1("%1_%2"))
         .arg(daykey)
         .arg((*event).todoUid());
-        
+
     if (taskdaytotals.contains(daytaskkey))
-        taskdaytotals.replace(daytaskkey, 
+        taskdaytotals.replace(daytaskkey,
                 taskdaytotals[daytaskkey] + (*event).duration());
     else
         taskdaytotals.insert(daytaskkey, (*event).duration());
   }
-        
+
   // day headings
   dayheading = from;
   while ( dayheading <= to )
@@ -923,7 +960,7 @@ QString KarmStorage::exportcsvHistory ( TaskView      *taskview,
   retval += i18n("Sum") + delim + i18n("Total Sum") + delim + i18n("Task Hierarchy");
   retval += cr;
   retval += line;
-        
+
   // the tasks
   vector <QString> matrix;
   linenr=0;
@@ -934,30 +971,30 @@ QString KarmStorage::exportcsvHistory ( TaskView      *taskview,
   }
   else
   {
-    if ( rc.allTasks ) 
+    if ( rc.allTasks )
     {
       for ( Task* task= taskview->item_at_index(0);
             task; task= task->nextSibling() )
       {
-        printTaskHistory( task, taskdaytotals, daytotals, from, to, 0, 
+        printTaskHistory( task, taskdaytotals, daytotals, from, to, 0,
                           matrix, rc );
       }
     }
     else
     {
-      printTaskHistory( taskview->current_item(), taskdaytotals, daytotals, 
+      printTaskHistory( taskview->current_item(), taskdaytotals, daytotals,
                         from, to, 0, matrix, rc );
     }
     for (unsigned int i=0; i<matrix.size(); i++) retval+=matrix[i];
     retval += line;
-        
+
     // totals
     sum = 0;
     day = from;
     while (day<=to)
     {
       daykey = day.toString(QString::fromLatin1("yyyyMMdd"));
-        
+
       if (daytotals.contains(daykey))
       {
         retval += QString::fromLatin1("%1")
@@ -967,7 +1004,7 @@ QString KarmStorage::exportcsvHistory ( TaskView      *taskview,
       retval += delim;
       day = day.addDays(1);
     }
-        
+
     retval += QString::fromLatin1("%1%2%3%4")
         .arg( formatTime( sum/60, rc.decimalMinutes ) )
         .arg( delim ).arg( delim )
@@ -975,11 +1012,11 @@ QString KarmStorage::exportcsvHistory ( TaskView      *taskview,
   }
 
   // above taken from timekard.cpp
-  
+
   // save, either locally or remote
-  
+
   if ((rc.url.isLocalFile()) || (!rc.url.url().contains("/")))
-  {    
+  {
     QString filename=rc.url.path();
     if (filename.isEmpty()) filename=rc.url.url();
     QFile f( filename );
@@ -997,26 +1034,51 @@ QString KarmStorage::exportcsvHistory ( TaskView      *taskview,
   else // use remote file
   {
     KTempFile tmpFile;
-    if ( tmpFile.status() != 0 ) err = QString::fromLatin1( "Unable to get temporary file" ); 
+    if ( tmpFile.status() != 0 )
+    {
+      err = QString::fromLatin1( "Unable to get temporary file" );
+    }
     else
     {
       QTextStream *stream=tmpFile.textStream();
       *stream << retval;
-      tmpFile.close(); 
+      tmpFile.close();
       if (!KIO::NetAccess::upload( tmpFile.name(), rc.url, 0 )) err=QString::fromLatin1("Could not upload");
     }
   }
   return err;
 }
 
-void KarmStorage::stopTimer(const Task* task)
+void KarmStorage::stopTimer(const Task* task, QDateTime when)
 {
-  long delta = task->startTime().secsTo(QDateTime::currentDateTime());
+  kdDebug(5970) << "Entering KarmStorage::stopTimer" << endl;
+  long delta = task->startTime().secsTo(when);
   changeTime(task, delta);
 }
 
+bool KarmStorage::bookTime(const Task* task,
+                           const QDateTime& startDateTime,
+                           const long durationInSeconds)
+{
+  // Ignores preferences setting re: logging history.
+  KCal::Event* e;
+  QDateTime end;
+
+  e = baseEvent( task );
+  e->setDtStart( startDateTime );
+  e->setDtEnd( startDateTime.addSecs( durationInSeconds ) );
+
+  // Use a custom property to keep a record of negative durations
+  e->setCustomProperty( kapp->instanceName(),
+      QCString("duration"),
+      QString::number(durationInSeconds));
+
+  return _calendar->addEvent(e);
+}
+
 void KarmStorage::changeTime(const Task* task, const long deltaSeconds)
 {
+  kdDebug(5970) << "Entering KarmStorage::changeTime ( " << task->name() << "," << deltaSeconds <<  " )" << endl;
   KCal::Event* e;
   QDateTime end;
 
@@ -1099,7 +1161,7 @@ QValueList<HistoryEvent> KarmStorage::getHistory(const QDate& from,
 
   for(QDate d = from; d <= to; d = d.addDays(1))
   {
-    events = _calendar->events(d);
+    events = _calendar->rawEventsForDate( d );
     for (event = events.begin(); event != events.end(); ++event)
     {
 
@@ -1148,64 +1210,32 @@ bool KarmStorage::remoteResource( const QString& file ) const
 {
   QString f = file.lower();
   bool rval = f.startsWith( "http://" ) || f.startsWith( "ftp://" );
-  
+
   kdDebug(5970) << "KarmStorage::remoteResource( " << file << " ) returns " << rval  << endl;
   return rval;
 }
 
-/*
- * Obsolete methods for writing to flat file format.
- * Aug 8, 2003, Mark
- *
-void KarmStorage::saveToFileFormat()
+bool KarmStorage::saveCalendar()
 {
-  //QFile f(_preferences->saveFile());
-  QFile f(_preferences->flatFile());
+  kdDebug(5970) << "KarmStorage::saveCalendar" << endl;
 
-  if ( !f.open( IO_WriteOnly | IO_Truncate ) ) {
-    QString msg = i18n( "There was an error trying to save your data file.\n"
-                       "Time accumulated during this session will not be saved!\n");
-    KMessageBox::error(0, msg );
-    return;
+#if 0
+  Event::List evl=_calendar->rawEvents();
+  kdDebug(5970) << "summary - dtStart - dtEnd" << endl;
+  for (unsigned int i=0; i<evl.count(); i++) 
+  {
+    kdDebug() << evl[i]->summary() << evl[i]->dtStart() << evl[i]->dtEnd() << endl;
   }
-  const char * comment = "# TaskView save data\n";
+#endif
+  KABC::Lock *lock = _calendar->lock();
+  if ( !lock || !lock->lock() )
+    return false;
 
-  f.writeBlock(comment, strlen(comment));  //comment
-  f.flush();
+  if ( _calendar && _calendar->save() ) {
+    lock->unlock();
+    return true;
+  }
 
-  QTextStream stream(&f);
-  for (Task* child = firstChild();
-             child;
-             child = child->nextSibling())
-    writeTaskToFile(&stream, child, 1);
-
-  f.close();
-  kdDebug(5970) << "Saved data to file " << f.name() << endl;
+  lock->unlock();
+  return false;
 }
-void KarmStorage::writeTaskToFile( QTextStream *strm, Task *task,
-                                int level)
-{
-  //lukas: correct version for non-latin1 users
-  QString _line = QString::fromLatin1("%1\t%2\t%3").arg(level).
-          arg(task->time()).arg(task->name());
-
-  DesktopList d = task->getDesktops();
-  int dsize = d.size();
-  if (dsize>0) {
-    _line += '\t';
-    for (int i=0; i<dsize-1; i++) {
-      _line += QString::number(d[i]);
-      _line += ',';
-    }
-    _line += QString::number(d[dsize-1]);
-  }
-  *strm << _line << "\n";
-
-  for ( Task* child= task->firstChild();
-              child;
-              child=child->nextSibling()) {
-    writeTaskToFile(strm, child, level+1);
-  }
-}
-
-*/
