@@ -1,10 +1,10 @@
 /* -*- Mode: C++ -*-
-   $Id: KDGanttView.h,v 1.4 2004/03/21 13:34:16 rogowski Exp $
+   $Id: KDGanttView.h 837187 2008-07-24 05:48:39Z pradeepto $
    KDChart - a multi-platform charting engine
 */
 
 /****************************************************************************
- ** Copyright (C)  2001-2004 Klarlvdalens Datakonsult AB.  All rights reserved.
+ ** Copyright (C)  2001-2004 Klarälvdalens Datakonsult AB.  All rights reserved.
  **
  ** This file is part of the KDGantt library.
  **
@@ -50,6 +50,7 @@
 #include "KDGanttMinimizeSplitter.h"
 #include "KDGanttViewItemDrag.h"
 
+class KDIntervalColorRectangle;
 class KDGanttViewTaskLink;
 class QPrinter;
 class QIODevice;
@@ -259,6 +260,8 @@ public:
                                    const QColor& color,
 				   Scale mini =  KDGanttView::Minute ,
 				   Scale maxi =  KDGanttView::Month);
+#if 0
+   // This API has been replaced with KDIntervalColorRectangle and addIntervalBackgroundColor
     void setIntervalBackgroundColor( const QDateTime& start,
 				     const QDateTime& end,
 				     const QColor& color,
@@ -270,6 +273,8 @@ public:
 				   const QDateTime& newend );
     bool deleteBackgroundInterval( const QDateTime& start,
 				   const QDateTime& end );
+#endif
+    void addIntervalBackgroundColor( KDIntervalColorRectangle* newItem );
     void clearBackgroundColor();
     QColor columnBackgroundColor( const QDateTime& column ) const;
     void setWeekendBackgroundColor( const QColor& color );
@@ -324,7 +329,7 @@ public:
     virtual void lvStartDrag (KDGanttViewItem*);
     virtual bool lvDragMoveEvent (QDragMoveEvent * e,KDGanttViewItem*, KDGanttViewItem*);
     virtual void lvDragEnterEvent (QDragEnterEvent * e );
-    QSize sizeHint();
+    virtual QSize sizeHint() const;
     KDGanttViewItem* getItemByName( const QString& name ) const;
     QDateTime getDateTimeForCoordX(int coordX, bool global = true ) const;
     KDGanttViewItem* getItemByListViewPos( const QPoint& pos ) const;
@@ -336,6 +341,14 @@ public:
     void setLvVScrollBarMode( QScrollView::ScrollBarMode );
     void setGvVScrollBarMode( QScrollView::ScrollBarMode );
 
+    void setLinkItemsEnabled(bool on);
+    bool isLinkItemsEnabled() const;
+
+    KDTimeTableWidget * timeTableWidget() { return myTimeTable; }
+    KDTimeHeaderWidget * timeHeaderWidget() { return myTimeHeader; }
+
+    void setFixedHorizon( bool f ) { mFixedHorizon = f; }
+
 public slots:
     void editItem( KDGanttViewItem* );
     void zoomToSelection( const QDateTime& start,  const QDateTime&  end);
@@ -343,7 +356,8 @@ public slots:
 signals:
     void timeIntervallSelected( const QDateTime& start,  const QDateTime&  end);
     void timeIntervalSelected( const QDateTime& start,  const QDateTime&  end);
-    void rescaling( Scale );
+    void rescaling( KDGanttView::Scale );
+    void intervalColorRectangleMoved( const QDateTime& start, const QDateTime& end );
 
     // the following signals are emitted if an item is clicked in the
     // listview (inclusive) or in the ganttview
@@ -352,6 +366,9 @@ signals:
     void itemRightClicked( KDGanttViewItem* );
     void itemDoubleClicked( KDGanttViewItem* );
 
+    // The following signal is emitted when two items shall be linked
+    void linkItems( KDGanttViewItem* from, KDGanttViewItem* to, int linkType );
+
     // the following signals are emitted if an item is clicked in the
     // listview (exlusive) or in the ganttview
     // gv... means item in ganttview clicked
@@ -365,6 +382,7 @@ signals:
     void gvItemDoubleClicked( KDGanttViewItem* );
     // the point is the global position!!
     void gvContextMenuRequested ( KDGanttViewItem * item, const QPoint & pos );
+    void gvItemMoved( KDGanttViewItem* );
 
     // lv... means item in listview clicked
     void lvCurrentChanged( KDGanttViewItem* );
@@ -385,16 +403,17 @@ signals:
     void taskLinkRightClicked( KDGanttViewTaskLink* );
     void taskLinkDoubleClicked( KDGanttViewTaskLink* );
 
+    void dateTimeDoubleClicked (const QDateTime& );
+
     void dropped ( QDropEvent * e, KDGanttViewItem* droppedItem, KDGanttViewItem* itemBelowMouse);
 private slots:
     void forceRepaint( int val = 0 );
     void slotSelectionChanged( QListViewItem* item );
     void slotCurrentChanged ( QListViewItem * item );
     void slotItemRenamed ( QListViewItem * item, int col, const QString & text  );
-  void slotMouseButtonPressed (  int button, QListViewItem * item, const QPoint & pos, int c );
+    void slotMouseButtonPressed ( int button, QListViewItem * item, const QPoint & pos, int c );
     void slotmouseButtonClicked ( int button, QListViewItem * item, const QPoint & pos, int c );
     void slotcontextMenuRequested ( QListViewItem * item, const QPoint & pos, int col );
-    void slotdoubleClicked ( QListViewItem * item );
     void slotHeaderSizeChanged();
     void addTickRight();
     void addTickLeft();
@@ -409,6 +428,8 @@ private:
     bool loadXML( const QDomDocument& doc );
     QDomDocument saveXML( bool withPI = true ) const;
 
+    void emptySpaceDoubleClicked( QMouseEvent* e );
+
     static QString scaleToString( Scale scale );
     static QString yearFormatToString( YearFormat format );
     static QString hourFormatToString( HourFormat format );
@@ -470,6 +491,7 @@ private:
     void initDefaults();
     KDGanttViewItem* myCurrentItem;
     KDGanttMinimizeSplitter *mySplitter;
+    bool  mFixedHorizon;
 protected:
   virtual QDragObject * dragObject ();
   virtual void startDrag ();
