@@ -1,10 +1,10 @@
 /* -*- Mode: C++ -*-
-   $Id: KDGanttViewItem.cpp,v 1.4 2004/03/21 13:34:17 rogowski Exp $
+   $Id: KDGanttViewItem.cpp 766119 2008-01-25 10:19:31Z tilladam $
    KDGantt - a multi-platform charting engine
 */
 
 /****************************************************************************
- ** Copyright (C)  2002-2004 Klarlvdalens Datakonsult AB.  All rights reserved.
+ ** Copyright (C)  2002-2004 Klarälvdalens Datakonsult AB.  All rights reserved.
  **
  ** This file is part of the KDGantt library.
  **
@@ -260,6 +260,9 @@ KDGanttViewItem::~KDGanttViewItem()
   if ( startShapeBack ) delete startShapeBack  ;
   if ( midShapeBack ) delete midShapeBack   ;
   if ( endShapeBack ) delete endShapeBack  ;
+  if ( progressShape ) delete  progressShape ;
+  if ( floatStartShape ) delete  floatStartShape ;
+  if ( floatEndShape ) delete  floatEndShape ;
   myGanttView->myTimeTable->removeItemFromTasklinks( this );
   myGanttView->myCanvasView->resetCutPaste( this );
   if ( listView() ) {
@@ -280,19 +283,23 @@ KDGanttViewItem::~KDGanttViewItem()
   Generates a unique name if necessary and inserts it into the item
   dictionary.
 */
-void KDGanttViewItem::generateAndInsertName( const QString& /*name*/ )
+void KDGanttViewItem::generateAndInsertName( const QString& name )
 {
     // First check if we already had a name. This can be the case if
     // the item was reconstructed from an XML file.
     if( !_name.isEmpty() )
         // We had a name, remove it
         sItemDict.remove( _name );
-    QString newName;
-    // create unique name
-    newName.sprintf( "%p", (void* )this );
 
-    while( sItemDict.find( newName ) ) {
-      newName += "_0";
+    QString newName;
+    if ( name.isEmpty() || sItemDict.find( name ) ) {
+        // create unique name
+        newName.sprintf( "%p", (void* )this );
+        while( sItemDict.find( newName ) ) {
+            newName += "_0";
+        }
+    } else {
+        newName = name;
     }
     sItemDict.insert( newName, this );
     _name = newName;
@@ -320,6 +327,8 @@ QString KDGanttViewItem::name() const
 
 KDGanttViewItem* KDGanttViewItem::find( const QString& name )
 {
+    if (name.isEmpty()) // avoid error msg from QDict
+        return 0;
     return sItemDict.find( name );
 }
 
@@ -1412,9 +1421,16 @@ void KDGanttViewItem::setCallListViewOnSetOpen( bool call )
 
 void KDGanttViewItem::initColorAndShapes(Type t)
 {
+    _isMoveable = false;
+    _isResizeable = false;
+    setTextOffset(QPoint(0,0));
   //_isCalendar = false;
     _callListViewOnSetOpen = true;
     myType = t;
+    myProgress = 0;
+    progressShape = 0;
+    floatStartShape = 0;
+    floatEndShape = 0;
     blockUpdating = true;
     isVisibleInGanttView = false;
     startShape = 0;
@@ -1465,7 +1481,10 @@ void KDGanttViewItem::initColorAndShapes(Type t)
       if ( startShape )
 	delete startShape;
       startShape = (KDCanvasPolygonItem*)new  KDCanvasRectangle(myGanttView->myTimeTable,this,Type_is_KDGanttViewItem);
+      progressShape = (KDCanvasPolygonItem*)new  KDCanvasRectangle(myGanttView->myTimeTable,this,Type_is_KDGanttViewItem);
     }
+    floatStartShape = (KDCanvasPolygonItem*)new  KDCanvasRectangle(myGanttView->myTimeTable,this,Type_is_KDGanttViewItem);
+    floatEndShape = (KDCanvasPolygonItem*)new  KDCanvasRectangle(myGanttView->myTimeTable,this,Type_is_KDGanttViewItem);
 
     // set color of shapes
     if (!( colorDefined = (myGanttView->colors(myType,myStartColor,myMiddleColor,myEndColor)))) {
@@ -2441,3 +2460,112 @@ bool KDGanttViewItem::isCalendar( ) const
   this instance variable is true if the item is visible in the Gantt
   view
 */
+
+
+/*!
+  Returns the coordinate of this items middle left point
+*/
+QPoint KDGanttViewItem::middleLeft()
+{
+    return QPoint(myGanttView->myTimeHeader->getCoordX(myStartTime), itemPos()+height()/2);
+}
+/*!
+  Returns the coordinate of this items middle right point
+*/
+QPoint KDGanttViewItem::middleRight()
+{
+    return QPoint(myGanttView->myTimeHeader->getCoordX(myEndTime), itemPos()+height()/2);
+}
+/*!
+  Moves this items text.
+*/
+void KDGanttViewItem::moveTextCanvas(int x, int y)
+{
+    int mx = x + myTextOffset.x();
+    int my = y + myTextOffset.y();
+    if (myTextOffset.x() != 0)
+        mx -= 2*myItemSize; // keep old behaviour
+
+    textCanvas->move(mx+2*myItemSize,my-myItemSize/2);
+    //qDebug("%s: moveTextCanvas(%d,%d) offset: %d,%d moved to  %d,%d",listViewText(0).latin1(),x,y,myTextOffset.x(),myTextOffset.y(),mx+2*myItemSize,my-myItemSize/2);
+}
+
+/*!
+  Moves this items text relative to the middle right end of the item
+  Used to move text away from link.
+*/
+void KDGanttViewItem::moveTextCanvas()
+{
+    QPoint m = myTextOffset+middleRight();
+    textCanvas->move(m.x(), m.y()-myItemSize/2);
+}
+
+/*!
+  Sets with how much the item text is offset.
+*/
+void KDGanttViewItem::setTextOffset(QPoint p)
+{
+    //qDebug("%s: setTextOffset() offset: %d,%d",listViewText(0).latin1(),p.x(),p.y());
+    myTextOffset.setX(p.x());
+    myTextOffset.setY(p.y());
+}
+
+bool KDGanttViewItem::isMyTextCanvas(QCanvasItem *tc)
+{
+    return tc == textCanvas;
+}
+
+/*!
+  Specifies the progress of this item in percent.
+  Progress is limited to minimum 0, maximum 100.
+
+  \param percent the progress in percent.
+*/
+
+void KDGanttViewItem::setProgress(int percent)
+{
+    myProgress = QMAX(0, percent);
+    myProgress = QMIN(100, myProgress);
+}
+
+/*!
+  Specifies the float start time of this item.
+  If the time is invalid, the start float is not shown.
+
+  \param start the float start time
+*/
+void KDGanttViewItem::setFloatStartTime(const QDateTime &start)
+{
+    myFloatStartTime = start;
+}
+
+/*!
+  Specifies the float end time of this item.
+  If the time is invalid, the end float is not shown.
+
+  \param end the float end time
+*/
+void KDGanttViewItem::setFloatEndTime(const QDateTime &end)
+{
+    myFloatEndTime = end;
+}
+
+void KDGanttViewItem::setMoveable(bool m)
+{
+  _isMoveable = m;
+}
+
+bool KDGanttViewItem::isMoveable() const
+{
+  return _isMoveable;
+}
+
+bool KDGanttViewItem::isResizeable() const
+{
+  return _isResizeable;
+}
+
+void KDGanttViewItem::setResizeable(bool r)
+{
+  _isResizeable = r;
+}
