@@ -10,6 +10,11 @@
 // other KMail headers:
 #include "kmsearchpatternedit.h"
 #include "kmfiltermgr.h"
+#include "kmmainwidget.h"
+#include "accountmanager.h"
+using KMail::AccountManager;
+#include "filterimporterexporter.h"
+using KMail::FilterImporterExporter;
 
 // other KDE headers:
 #include <kmessagebox.h>
@@ -21,6 +26,9 @@
 #include <kwin.h>
 #include <kconfig.h>
 #include <kicondialog.h>
+#include <kkeybutton.h>
+#include <klistview.h>
+#include <kpushbutton.h>
 
 // other Qt headers:
 #include <qlayout.h>
@@ -30,8 +38,9 @@
 #include <qtooltip.h>
 #include <qwhatsthis.h>
 #include <qcheckbox.h>
-#include <qpushbutton.h>
 #include <qhbox.h>
+#include <qvalidator.h>
+#include <qtabwidget.h>
 
 // other headers:
 #include <assert.h>
@@ -62,6 +71,12 @@ I18N_NOOP( "<qt><p>Click this button to <em>delete</em> the currently-"
 	   "it is deleted, but you can always leave the "
 	   "dialog by clicking <em>Cancel</em> to discard the "
 	   "changes made.</p></qt>" );
+const char * _wt_filterlist_top =
+I18N_NOOP( "<qt><p>Click this button to move the currently-"
+	   "selected filter to the <em>top</em> of the list above.</p>"
+	   "<p>This is useful since the order of the filters in the list "
+	   "determines the order in which they are tried on messages: "
+	   "The topmost filter gets tried first.</p></qt>" );
 const char * _wt_filterlist_up =
 I18N_NOOP( "<qt><p>Click this button to move the currently-"
 	   "selected filter <em>up</em> one in the list above.</p>"
@@ -78,10 +93,16 @@ I18N_NOOP( "<qt><p>Click this button to move the currently-"
 	   "The topmost filter gets tried first.</p>"
 	   "<p>If you have clicked this button accidentally, you can undo this "
 	   "by clicking on the <em>Up</em> button.</p></qt>" );
+const char * _wt_filterlist_bot =
+I18N_NOOP( "<qt><p>Click this button to move the currently-"
+	   "selected filter to the <em>bottom</em> of the list above.</p>"
+	   "<p>This is useful since the order of the filters in the list "
+	   "determines the order in which they are tried on messages: "
+	   "The topmost filter gets tried first.</p></qt>" );
 const char * _wt_filterlist_rename =
 I18N_NOOP( "<qt><p>Click this button to rename the currently-selected filter.</p>"
 	   "<p>Filters are named automatically, as long as they start with "
-	   "\"<<\".</p>"
+	   "\"&lt;\".</p>"
 	   "<p>If you have renamed a filter accidentally and want automatic "
 	   "naming back, click this button and select <em>Clear</em> followed "
 	   "by <em>OK</em> in the appearing dialog.</p></qt>" );
@@ -103,27 +124,51 @@ const char * KMPopFilterDlgHelpAnchor =  "popfilters-id" ;
 //
 //=============================================================================
 
-KMFilterDlg::KMFilterDlg(QWidget* parent, const char* name, bool popFilter)
-  : KDialogBase( parent, name, FALSE /* modality */,
+KMFilterDlg::KMFilterDlg(QWidget* parent, const char* name, bool popFilter, bool createDummyFilter )
+  : KDialogBase( parent, name,  false  /* modality */,
 		 (popFilter)? i18n("POP3 Filter Rules"): i18n("Filter Rules") /* caption*/,
-		 Help|Ok|Apply|Cancel /* button mask */,
-		 Ok /* default btn */, FALSE /* separator */),
+		 Help|Ok|Apply|Cancel|User1|User2 /* button mask */,
+		 Ok /* default btn */,  false  /* separator */),
   bPopFilter(popFilter)
 {
   KWin::setIcons( winId(), kapp->icon(), kapp->miniIcon() );
   setHelp( (bPopFilter)? KMPopFilterDlgHelpAnchor: KMFilterDlgHelpAnchor );
+  setButtonText( User1, i18n("Import") );
+  setButtonText( User2, i18n("Export") );
+  connect( this, SIGNAL(user1Clicked()),
+           this, SLOT( slotImportFilters()) );
+  connect( this, SIGNAL(user2Clicked()),
+           this, SLOT( slotExportFilters()) );
 
-  QWidget *w = new QWidget(this);
-  setMainWidget(w);
-  QHBoxLayout *hbl = new QHBoxLayout( w, 0, spacingHint(), "kmfd_hbl" );
+  QWidget *w = new QWidget( this );
+  setMainWidget( w );
+  QHBoxLayout *topLayout = new QHBoxLayout( w, 0, spacingHint(), "topLayout" );
+  QHBoxLayout *hbl = topLayout;
+  QVBoxLayout *vbl2 = 0;
+  QWidget *page1 = 0;
+  QWidget *page2 = 0;
 
   mFilterList = new KMFilterListBox( i18n("Available Filters"), w, 0, bPopFilter);
-  hbl->addWidget( mFilterList, 1 /*stretch*/ );
+  topLayout->addWidget( mFilterList, 1 /*stretch*/ );
+
+  if(!bPopFilter) {
+    QTabWidget *tabWidget = new QTabWidget( w, "kmfd_tab" );
+    tabWidget->setMargin( KDialog::marginHint() );
+    topLayout->addWidget( tabWidget );
+
+    page1 = new QWidget( tabWidget );
+    tabWidget->addTab( page1, i18n("&General") );
+    hbl = new QHBoxLayout( page1, 0, spacingHint(), "kmfd_hbl" );
+
+    page2 = new QWidget( tabWidget );
+    tabWidget->addTab( page2, i18n("A&dvanced") );
+    vbl2 = new QVBoxLayout( page2, 0, spacingHint(), "kmfd_vbl2" );
+  }
 
   QVBoxLayout *vbl = new QVBoxLayout( hbl, spacingHint(), "kmfd_vbl" );
   hbl->setStretchFactor( vbl, 2 );
 
-  mPatternEdit = new KMSearchPatternEdit( i18n("Filter Criteria"), w , "spe", bPopFilter);
+  mPatternEdit = new KMSearchPatternEdit( i18n("Filter Criteria"), bPopFilter ? w : page1 , "spe", bPopFilter);
   vbl->addWidget( mPatternEdit, 0, Qt::AlignTop );
 
   if(bPopFilter){
@@ -136,31 +181,63 @@ KMFilterDlg::KMFilterDlg(QWidget* parent, const char* name, bool popFilter)
     vbl->addWidget( mGlobalsBox, 0, Qt::AlignTop );
   }
   else {
-    QGroupBox *agb = new QGroupBox( 1 /*column*/, Vertical, i18n("Filter Actions"), w );
+    QGroupBox *agb = new QGroupBox( 1 /*column*/, Vertical, i18n("Filter Actions"), page1 );
     mActionLister = new KMFilterActionWidgetLister( agb );
     vbl->addWidget( agb, 0, Qt::AlignTop );
 
     mAdvOptsGroup = new QGroupBox ( 1 /*columns*/, Vertical,
-				    i18n("Advanced Options"), w);
+				    i18n("Advanced Options"), page2);
     {
       QWidget *adv_w = new QWidget( mAdvOptsGroup );
-      QGridLayout *gl = new QGridLayout( adv_w, 4 /*rows*/, 4 /*cols*/,
-				         0 /*border*/, spacingHint() );
-      gl->setColStretch( 0, 1 );
-      QLabel *l = new QLabel( i18n("Apply this filter"), adv_w );
-      gl->addWidget( l, 0, 0, AlignLeft );
-      mApplyOnIn = new QCheckBox( i18n("to &incoming messages"), adv_w );
-      gl->addWidget( mApplyOnIn, 0, 1 );
-      mApplyOnOut = new QCheckBox( i18n("to &sent messages"), adv_w );
-      gl->addWidget( mApplyOnOut, 0, 2 );
-      mApplyOnCtrlJ = new QCheckBox( i18n("on manual &filtering"), adv_w );
-      gl->addWidget( mApplyOnCtrlJ, 0, 3 );
+      QGridLayout *gl = new QGridLayout( adv_w, 8 /*rows*/, 3 /*cols*/,
+      				         0 /*border*/, spacingHint() );
+
+      QVBoxLayout *vbl3 = new QVBoxLayout( gl, spacingHint(), "vbl3" );
+      vbl3->addStretch( 1 );
+      mApplyOnIn = new QCheckBox( i18n("Apply this filter to incoming messages:"), adv_w );
+      vbl3->addWidget( mApplyOnIn );
+      QButtonGroup *bg = new QButtonGroup( 0, "bg" );
+      bg->setExclusive( true );
+      mApplyOnForAll = new QRadioButton( i18n("from all accounts"), adv_w );
+      bg->insert( mApplyOnForAll );
+      vbl3->addWidget( mApplyOnForAll );
+      mApplyOnForTraditional = new QRadioButton( i18n("from all but online IMAP accounts"), adv_w );
+      bg->insert( mApplyOnForTraditional );
+      vbl3->addWidget( mApplyOnForTraditional );
+      mApplyOnForChecked = new QRadioButton( i18n("from checked accounts only"), adv_w );
+      bg->insert( mApplyOnForChecked );
+      vbl3->addWidget( mApplyOnForChecked );
+      vbl3->addStretch( 2 );
+
+      mAccountList = new KListView( adv_w, "accountList" );
+      mAccountList->addColumn( i18n("Account Name") );
+      mAccountList->addColumn( i18n("Type") );
+      mAccountList->setAllColumnsShowFocus( true );
+      mAccountList->setFrameStyle( QFrame::WinPanel + QFrame::Sunken );
+      mAccountList->setSorting( -1 );
+      gl->addMultiCellWidget( mAccountList, 0, 3, 1, 3 );
+
+      mApplyOnOut = new QCheckBox( i18n("Apply this filter to &sent messages"), adv_w );
+      gl->addMultiCellWidget( mApplyOnOut, 4, 4, 0, 3 );
+
+      mApplyOnCtrlJ = new QCheckBox( i18n("Apply this filter on manual &filtering"), adv_w );
+      gl->addMultiCellWidget( mApplyOnCtrlJ, 5, 5, 0, 3 );
+
       mStopProcessingHere = new QCheckBox( i18n("If this filter &matches, stop processing here"), adv_w );
-      gl->addMultiCellWidget( mStopProcessingHere, //1, 0, Qt::AlignLeft );
-			      1, 1, /*from to row*/
+      gl->addMultiCellWidget( mStopProcessingHere,
+			      6, 6, /*from to row*/
   			      0, 3 /*from to col*/ );
       mConfigureShortcut = new QCheckBox( i18n("Add this filter to the Apply Filter menu"), adv_w );
-      gl->addMultiCellWidget( mConfigureShortcut, 2, 2, 0, 3 );
+      gl->addMultiCellWidget( mConfigureShortcut, 7, 7, 0, 1 );
+      QLabel *keyButtonLabel = new QLabel( i18n( "Shortcut:" ), adv_w );
+      keyButtonLabel->setAlignment( AlignVCenter | AlignRight );
+      gl->addMultiCellWidget( keyButtonLabel, 7, 7, 2, 2 );
+      mKeyButton = new KKeyButton( adv_w, "FilterShortcutSelector" );
+      gl->addMultiCellWidget( mKeyButton, 7, 7, 3, 3 );
+      mKeyButton->setEnabled( false );
+      mConfigureToolbar = new QCheckBox( i18n("Additionally add this filter to the toolbar"), adv_w );
+      gl->addMultiCellWidget( mConfigureToolbar, 8, 8, 0, 3 );
+      mConfigureToolbar->setEnabled( false );
 
       QHBox *hbox = new QHBox( adv_w );
       mFilterActionLabel = new QLabel( i18n( "Icon for this filter:" ),
@@ -174,9 +251,9 @@ KMFilterDlg::KMFilterDlg(QWidget* parent, const char* name, bool popFilter)
       mFilterActionIconButton->setIcon( "gear" );
       mFilterActionIconButton->setEnabled( false );
 
-      gl->addMultiCellWidget( hbox, 3, 3, 0, 3 );
+      gl->addMultiCellWidget( hbox, 9, 9, 0, 3 );
     }
-    vbl->addWidget( mAdvOptsGroup, 0, Qt::AlignTop );
+    vbl2->addWidget( mAdvOptsGroup, 0, Qt::AlignTop );
   }
   // spacer:
   vbl->addStretch( 1 );
@@ -198,10 +275,20 @@ KMFilterDlg::KMFilterDlg(QWidget* parent, const char* name, bool popFilter)
     // combo box to the filter
     connect( mApplyOnIn, SIGNAL(clicked()),
   	     this, SLOT(slotApplicabilityChanged()) );
+    connect( mApplyOnForAll, SIGNAL(clicked()),
+  	     this, SLOT(slotApplicabilityChanged()) );
+    connect( mApplyOnForTraditional, SIGNAL(clicked()),
+  	     this, SLOT(slotApplicabilityChanged()) );
+    connect( mApplyOnForChecked, SIGNAL(clicked()),
+  	     this, SLOT(slotApplicabilityChanged()) );
     connect( mApplyOnOut, SIGNAL(clicked()),
   	     this, SLOT(slotApplicabilityChanged()) );
     connect( mApplyOnCtrlJ, SIGNAL(clicked()),
   	     this, SLOT(slotApplicabilityChanged()) );
+    connect( mAccountList, SIGNAL(clicked(QListViewItem*)),
+  	     this, SLOT(slotApplicableAccountsChanged()) );
+    connect( mAccountList, SIGNAL(spacePressed(QListViewItem*)),
+  	     this, SLOT(slotApplicableAccountsChanged()) );
 
     // transfer changes from the 'stop processing here'
     // check box to the filter
@@ -211,6 +298,12 @@ KMFilterDlg::KMFilterDlg(QWidget* parent, const char* name, bool popFilter)
     connect( mConfigureShortcut, SIGNAL(toggled(bool)),
 	     this, SLOT(slotConfigureShortcutButtonToggled(bool)) );
 
+    connect( mKeyButton, SIGNAL( capturedShortcut( const KShortcut& ) ),
+             this, SLOT( slotCapturedShortcutChanged( const KShortcut& ) ) );
+
+    connect( mConfigureToolbar, SIGNAL(toggled(bool)),
+	     this, SLOT(slotConfigureToolbarButtonToggled(bool)) );
+
     connect( mFilterActionIconButton, SIGNAL( iconChanged( QString ) ),
              this, SLOT( slotFilterActionIconChanged( QString ) ) );
   }
@@ -251,7 +344,7 @@ KMFilterDlg::KMFilterDlg(QWidget* parent, const char* name, bool popFilter)
     adjustSize();
 
   // load the filter list (emits filterSelected())
-  mFilterList->loadFilterList();
+  mFilterList->loadFilterList( createDummyFilter );
 }
 
 void KMFilterDlg::slotFinished() {
@@ -275,7 +368,7 @@ void KMFilterDlg::slotFilterSelected( KMFilter* aFilter )
 
   if (bPopFilter){
     mActionGroup->setAction( aFilter->action() );
-    mGlobalsBox->setEnabled(true);
+    mGlobalsBox->setEnabled( true );
     mShowLaterBtn->setChecked(mFilterList->showLaterMsgs());
   } else {
     mActionLister->setActionList( aFilter->actions() );
@@ -298,17 +391,31 @@ void KMFilterDlg::slotFilterSelected( KMFilter* aFilter )
     // the filter! So make sure we have the correct values _before_ we
     // set the first one:
     const bool applyOnIn = aFilter->applyOnInbound();
+    const bool applyOnForAll = aFilter->applicability() == KMFilter::All;
+    const bool applyOnTraditional = aFilter->applicability() == KMFilter::ButImap;
     const bool applyOnOut = aFilter->applyOnOutbound();
     const bool applyOnExplicit = aFilter->applyOnExplicit();
     const bool stopHere = aFilter->stopProcessingHere();
     const bool configureShortcut = aFilter->configureShortcut();
+    const bool configureToolbar = aFilter->configureToolbar();
     const QString icon = aFilter->icon();
+    const KShortcut shortcut( aFilter->shortcut() );
 
     mApplyOnIn->setChecked( applyOnIn );
+    mApplyOnForAll->setEnabled( applyOnIn );
+    mApplyOnForTraditional->setEnabled( applyOnIn );
+    mApplyOnForChecked->setEnabled( applyOnIn );
+    mApplyOnForAll->setChecked( applyOnForAll );
+    mApplyOnForTraditional->setChecked( applyOnTraditional );
+    mApplyOnForChecked->setChecked( !applyOnForAll && !applyOnTraditional );
+    mAccountList->setEnabled( mApplyOnForChecked->isEnabled() && mApplyOnForChecked->isChecked() );
+    slotUpdateAccountList();
     mApplyOnOut->setChecked( applyOnOut );
     mApplyOnCtrlJ->setChecked( applyOnExplicit );
     mStopProcessingHere->setChecked( stopHere );
     mConfigureShortcut->setChecked( configureShortcut );
+    mKeyButton->setShortcut( shortcut, false );
+    mConfigureToolbar->setChecked( configureToolbar );
     mFilterActionIconButton->setIcon( icon );
   }
 }
@@ -324,6 +431,7 @@ void KMFilterDlg::slotReset()
   } else {
     mActionLister->reset();
     mAdvOptsGroup->setEnabled( false );
+    slotUpdateAccountList();
   }
 }
 
@@ -337,43 +445,126 @@ void KMFilterDlg::slotUpdateFilter()
 
 void KMFilterDlg::slotApplicabilityChanged()
 {
-  if ( !mFilter )
-    return;
+  if ( mFilter ) {
+    mFilter->setApplyOnInbound( mApplyOnIn->isChecked() );
+    mFilter->setApplyOnOutbound( mApplyOnOut->isChecked() );
+    mFilter->setApplyOnExplicit( mApplyOnCtrlJ->isChecked() );
+    if ( mApplyOnForAll->isChecked() )
+      mFilter->setApplicability( KMFilter::All );
+    else if ( mApplyOnForTraditional->isChecked() )
+      mFilter->setApplicability( KMFilter::ButImap );
+    else if ( mApplyOnForChecked->isChecked() )
+      mFilter->setApplicability( KMFilter::Checked );
 
-  mFilter->setApplyOnInbound( mApplyOnIn->isChecked() );
-  mFilter->setApplyOnOutbound( mApplyOnOut->isChecked() );
-  mFilter->setApplyOnExplicit( mApplyOnCtrlJ->isChecked() );
-  kdDebug(5006) << "KMFilterDlg: setting filter to be applied at "
-		<< ( mFilter->applyOnInbound() ? "incoming " : "" )
-		<< ( mFilter->applyOnOutbound() ? "outgoing " : "" )
-		<< ( mFilter->applyOnExplicit() ? "explicit CTRL-J" : "" )
-		<< endl;
+    mApplyOnForAll->setEnabled( mApplyOnIn->isChecked() );
+    mApplyOnForTraditional->setEnabled(  mApplyOnIn->isChecked() );
+    mApplyOnForChecked->setEnabled( mApplyOnIn->isChecked() );
+    mAccountList->setEnabled( mApplyOnForChecked->isEnabled() && mApplyOnForChecked->isChecked() );
+
+    // Advanced tab functionality - Update list of accounts this filter applies to
+    QListViewItemIterator it( mAccountList );
+    while ( it.current() ) {
+      QCheckListItem *item = dynamic_cast<QCheckListItem*>( it.current() );
+      if (item) {
+	int id = item->text( 2 ).toInt();
+	  item->setOn( mFilter->applyOnAccount( id ) );
+      }
+      ++it;
+    }
+
+    kdDebug(5006) << "KMFilterDlg: setting filter to be applied at "
+                  << ( mFilter->applyOnInbound() ? "incoming " : "" )
+                  << ( mFilter->applyOnOutbound() ? "outgoing " : "" )
+                  << ( mFilter->applyOnExplicit() ? "explicit CTRL-J" : "" )
+                  << endl;
+  }
+}
+
+void KMFilterDlg::slotApplicableAccountsChanged()
+{
+  if ( mFilter && mApplyOnForChecked->isEnabled() && mApplyOnForChecked->isChecked() ) {
+    // Advanced tab functionality - Update list of accounts this filter applies to
+    QListViewItemIterator it( mAccountList );
+    while ( it.current() ) {
+      QCheckListItem *item = dynamic_cast<QCheckListItem*>( it.current() );
+      if (item) {
+	int id = item->text( 2 ).toInt();
+	mFilter->setApplyOnAccount( id, item->isOn() );
+      }
+      ++it;
+    }
+  }
 }
 
 void KMFilterDlg::slotStopProcessingButtonToggled( bool aChecked )
 {
-  if ( !mFilter )
-    return;
-
-  mFilter->setStopProcessingHere( aChecked );
+  if ( mFilter )
+    mFilter->setStopProcessingHere( aChecked );
 }
 
 void KMFilterDlg::slotConfigureShortcutButtonToggled( bool aChecked )
 {
-  if ( !mFilter )
-    return;
+  if ( mFilter ) {
+    mFilter->setConfigureShortcut( aChecked );
+    mKeyButton->setEnabled( aChecked );
+    mConfigureToolbar->setEnabled( aChecked );
+    mFilterActionIconButton->setEnabled( aChecked );
+    mFilterActionLabel->setEnabled( aChecked );
+  }
+}
 
-  mFilter->setConfigureShortcut( aChecked );
-  mFilterActionIconButton->setEnabled( aChecked );
-  mFilterActionLabel->setEnabled( aChecked );
+void KMFilterDlg::slotCapturedShortcutChanged( const KShortcut& sc )
+{
+  KShortcut mySc(sc);
+  if ( mySc == mKeyButton->shortcut() ) return;
+  // FIXME work around a problem when reseting the shortcut via the shortcut dialog
+  // somehow the returned shortcut does not evaluate to true in KShortcut::isNull(),
+  // so we additionally have to check for an empty string
+  if ( mySc.isNull() || mySc.toString().isEmpty() )
+    mySc.clear();
+  if ( !mySc.isNull() && !( kmkernel->getKMMainWidget()->shortcutIsValid( mySc ) ) ) {
+    QString msg( i18n( "The selected shortcut is already used, "
+          "please select a different one." ) );
+    KMessageBox::sorry( this, msg );
+  } else {
+    mKeyButton->setShortcut( mySc, false );
+    if ( mFilter )
+      mFilter->setShortcut( mKeyButton->shortcut() );
+  }
+}
+
+void KMFilterDlg::slotConfigureToolbarButtonToggled( bool aChecked )
+{
+  if ( mFilter )
+    mFilter->setConfigureToolbar( aChecked );
 }
 
 void KMFilterDlg::slotFilterActionIconChanged( QString icon )
 {
-  if ( !mFilter )
-    return;
+  if ( mFilter )
+    mFilter->setIcon( icon );
+}
 
-  mFilter->setIcon( icon );
+void KMFilterDlg::slotUpdateAccountList()
+{
+  mAccountList->clear();
+  QListViewItem *top = 0;
+  for( KMAccount *a = kmkernel->acctMgr()->first(); a!=0;
+       a = kmkernel->acctMgr()->next() ) {
+    QCheckListItem *listItem =
+      new QCheckListItem( mAccountList, top, a->name(), QCheckListItem::CheckBox );
+    listItem->setText( 1, a->type() );
+    listItem->setText( 2, QString( "%1" ).arg( a->id() ) );
+    if ( mFilter )
+      listItem->setOn( mFilter->applyOnAccount( a->id() ) );
+    top = listItem;
+  }
+
+  QListViewItem *listItem = mAccountList->firstChild();
+  if ( listItem ) {
+    mAccountList->setCurrentItem( listItem );
+    mAccountList->setSelected( listItem, true );
+  }
 }
 
 //=============================================================================
@@ -386,7 +577,7 @@ KMFilterListBox::KMFilterListBox( const QString & title, QWidget *parent, const
   : QGroupBox( 1, Horizontal, title, parent, name ),
     bPopFilter(popFilter)
 {
-  mFilterList.setAutoDelete(TRUE);
+  mFilterList.setAutoDelete( true );
   mIdxSelItem = -1;
 
   //----------- the list box
@@ -397,18 +588,30 @@ KMFilterListBox::KMFilterListBox( const QString & title, QWidget *parent, const
   //----------- the first row of buttons
   QHBox *hb = new QHBox(this);
   hb->setSpacing(4);
-  mBtnUp = new QPushButton( QString::null, hb );
+  mBtnTop = new KPushButton( QString::null, hb );
+  mBtnTop->setAutoRepeat( true );
+  mBtnTop->setIconSet( BarIconSet( "top", KIcon::SizeSmall ) );
+  mBtnTop->setMinimumSize( mBtnTop->sizeHint() * 1.2 );
+  mBtnUp = new KPushButton( QString::null, hb );
   mBtnUp->setAutoRepeat( true );
-  mBtnUp->setPixmap( BarIcon( "up", KIcon::SizeSmall ) );
+  mBtnUp->setIconSet( BarIconSet( "up", KIcon::SizeSmall ) );
   mBtnUp->setMinimumSize( mBtnUp->sizeHint() * 1.2 );
-  mBtnDown = new QPushButton( QString::null, hb );
+  mBtnDown = new KPushButton( QString::null, hb );
   mBtnDown->setAutoRepeat( true );
-  mBtnDown->setPixmap( BarIcon( "down", KIcon::SizeSmall ) );
+  mBtnDown->setIconSet( BarIconSet( "down", KIcon::SizeSmall ) );
   mBtnDown->setMinimumSize( mBtnDown->sizeHint() * 1.2 );
+  mBtnBot = new KPushButton( QString::null, hb );
+  mBtnBot->setAutoRepeat( true );
+  mBtnBot->setIconSet( BarIconSet( "bottom", KIcon::SizeSmall ) );
+  mBtnBot->setMinimumSize( mBtnBot->sizeHint() * 1.2 );
+  QToolTip::add( mBtnTop, i18n("Top") );
   QToolTip::add( mBtnUp, i18n("Up") );
   QToolTip::add( mBtnDown, i18n("Down") );
+  QToolTip::add( mBtnBot, i18n("Bottom") );
+  QWhatsThis::add( mBtnTop, i18n(_wt_filterlist_top) );
   QWhatsThis::add( mBtnUp, i18n(_wt_filterlist_up) );
   QWhatsThis::add( mBtnDown, i18n(_wt_filterlist_down) );
+  QWhatsThis::add( mBtnBot, i18n(_wt_filterlist_bot) );
 
   //----------- the second row of buttons
   hb = new QHBox(this);
@@ -417,10 +620,10 @@ KMFilterListBox::KMFilterListBox( const QString & title, QWidget *parent, const
   mBtnNew->setPixmap( BarIcon( "filenew", KIcon::SizeSmall ) );
   mBtnNew->setMinimumSize( mBtnNew->sizeHint() * 1.2 );
   mBtnCopy = new QPushButton( QString::null, hb );
-  mBtnCopy->setPixmap( BarIcon( "editcopy", KIcon::SizeSmall ) );
+  mBtnCopy->setIconSet( BarIconSet( "editcopy", KIcon::SizeSmall ) );
   mBtnCopy->setMinimumSize( mBtnCopy->sizeHint() * 1.2 );
   mBtnDelete = new QPushButton( QString::null, hb );
-  mBtnDelete->setPixmap( BarIcon( "editdelete", KIcon::SizeSmall ) );
+  mBtnDelete->setIconSet( BarIconSet( "editdelete", KIcon::SizeSmall ) );
   mBtnDelete->setMinimumSize( mBtnDelete->sizeHint() * 1.2 );
   mBtnRename = new QPushButton( i18n("Rename..."), hb );
   QToolTip::add( mBtnNew, i18n("New") );
@@ -437,10 +640,14 @@ KMFilterListBox::KMFilterListBox( const QString & title, QWidget *parent, const
 	   this, SLOT(slotSelected(int)) );
   connect( mListBox, SIGNAL( doubleClicked ( QListBoxItem * )),
            this, SLOT( slotRename()) );
+  connect( mBtnTop, SIGNAL(clicked()),
+	   this, SLOT(slotTop()) );
   connect( mBtnUp, SIGNAL(clicked()),
 	   this, SLOT(slotUp()) );
   connect( mBtnDown, SIGNAL(clicked()),
 	   this, SLOT(slotDown()) );
+  connect( mBtnBot, SIGNAL(clicked()),
+	   this, SLOT(slotBottom()) );
   connect( mBtnNew, SIGNAL(clicked()),
 	   this, SLOT(slotNew()) );
   connect( mBtnCopy, SIGNAL(clicked()),
@@ -486,7 +693,11 @@ void KMFilterListBox::slotUpdateFilterName()
   QString shouldBeName = p->name();
   QString displayedName = mListBox->text( mIdxSelItem );
 
-  if ( shouldBeName.stripWhiteSpace().isEmpty() || shouldBeName[0] == '<' ) {
+  if ( shouldBeName.stripWhiteSpace().isEmpty() ) {
+    mFilterList.at(mIdxSelItem)->setAutoNaming( true );
+  }
+
+  if ( mFilterList.at(mIdxSelItem)->isAutoNaming() ) {
     // auto-naming of patterns
     if ( p->first() && !p->first()->field().stripWhiteSpace().isEmpty() )
       shouldBeName = QString( "<%1>: %2" ).arg( p->first()->field() ).arg( p->first()->contents() );
@@ -497,9 +708,9 @@ void KMFilterListBox::slotUpdateFilterName()
 
   if ( displayedName == shouldBeName ) return;
 
-  mListBox->blockSignals(TRUE);
+  mListBox->blockSignals( true );
   mListBox->changeItem( shouldBeName, mIdxSelItem );
-  mListBox->blockSignals(FALSE);
+  mListBox->blockSignals( false );
 }
 
 void KMFilterListBox::slotShowLaterToggled(bool aOn)
@@ -509,14 +720,10 @@ void KMFilterListBox::slotShowLaterToggled(bool aOn)
 
 void KMFilterListBox::slotApplyFilterChanges()
 {
-  int oIdxSelItem = mIdxSelItem;
-  // unselect all filters:
-  mListBox->selectAll( FALSE );
-  // maybe QListBox doesn't emit selected(-1) on unselect,
-  // so we make sure the edit widgets receive an equivalent:
-  emit resetWidgets();
-  mIdxSelItem = -1;
-  enableControls();
+  if ( mIdxSelItem >= 0 ) {
+    emit applyWidgets();
+    slotSelected( mListBox->currentItem() );
+  }
 
   // by now all edit widgets should have written back
   // their widget's data into our filter list.
@@ -527,45 +734,50 @@ void KMFilterListBox::slotApplyFilterChanges()
   else
     fm = kmkernel->filterMgr();
 
-  // block attemts to use filters (currently a no-op)
-  fm->beginUpdate();
-  fm->clear();
+  QValueList<KMFilter*> newFilters = filtersForSaving();
 
-  QStringList emptyFilters;
-  QPtrListIterator<KMFilter> it( mFilterList );
-  for ( it.toFirst() ; it.current() ; ++it ) {
-    KMFilter *f = new KMFilter( **it ); // deep copy
-    f->purify();
-    if ( !f->isEmpty() )
-      // the filter is valid:
-      fm->append( f );
-    else {
-      // the filter is invalid:
-      emptyFilters << f->name();
-      delete f;
-    }
-  }
   if (bPopFilter)
     fm->setShowLaterMsgs(mShowLater);
 
-  // allow usage of the filters again.
-  fm->endUpdate();
-  fm->writeConfig();
-
-  // report on invalid filters:
-  if ( !emptyFilters.empty() ) {
-    QString msg = i18n("The following filters have not been saved because they "
-		       "were invalid (e.g. containing no actions or no search "
-		       "rules).");
-    KMessageBox::informationList( 0, msg, emptyFilters, QString::null,
-				  "ShowInvalidFilterWarning" );
+  fm->setFilters( newFilters );
+  if (fm->atLeastOneOnlineImapFolderTarget()) {
+    QString str = i18n("At least one filter targets a folder on an online "
+		       "IMAP account. Such filters will only be applied "
+		       "when manually filtering and when filtering "
+		       "incoming online IMAP mail.");
+    KMessageBox::information( this, str, QString::null,
+			      "filterDlgOnlineImapCheck" );
   }
+}
 
-  if ( oIdxSelItem >= 0 ) {
-    mIdxSelItem = oIdxSelItem;
-    mListBox->setSelected( oIdxSelItem, TRUE);
-    slotSelected( mListBox->currentItem() );
-  }
+QValueList<KMFilter*> KMFilterListBox::filtersForSaving() const
+{
+      const_cast<KMFilterListBox*>( this )->applyWidgets(); // signals aren't const
+      QValueList<KMFilter*> filters;
+      QStringList emptyFilters;
+      QPtrListIterator<KMFilter> it( mFilterList );
+      for ( it.toFirst() ; it.current() ; ++it ) {
+        KMFilter *f = new KMFilter( **it ); // deep copy
+        f->purify();
+        if ( !f->isEmpty() )
+          // the filter is valid:
+          filters.append( f );
+        else {
+          // the filter is invalid:
+          emptyFilters << f->name();
+          delete f;
+        }
+      }
+
+      // report on invalid filters:
+      if ( !emptyFilters.empty() ) {
+        QString msg = i18n("The following filters have not been saved because they "
+                   "were invalid (e.g. containing no actions or no search "
+                   "rules).");
+        KMessageBox::informationList( 0, msg, emptyFilters, QString::null,
+                      "ShowInvalidFilterWarning" );
+      }
+      return filters;
 }
 
 void KMFilterListBox::slotSelected( int aIdx )
@@ -618,7 +830,7 @@ void KMFilterListBox::slotDelete()
   int oIdxSelItem = mIdxSelItem;
   mIdxSelItem = -1;
   // unselect all
-  mListBox->selectAll(FALSE);
+  mListBox->selectAll( false );
   // broadcast that all widgets let go
   // of the filter
   emit resetWidgets();
@@ -632,16 +844,31 @@ void KMFilterListBox::slotDelete()
   // and set the new current item.
   if ( count > oIdxSelItem )
     // oIdxItem is still a valid index
-    mListBox->setSelected( oIdxSelItem, TRUE );
-  else if ( (int)mListBox->count() )
+    mListBox->setSelected( oIdxSelItem, true );
+  else if ( count )
     // oIdxSelIdx is no longer valid, but the
     // list box isn't empty
-    mListBox->setSelected( count - 1, TRUE );
+    mListBox->setSelected( count - 1, true );
   // the list is empty - keep index -1
 
   enableControls();
 }
 
+void KMFilterListBox::slotTop()
+{
+  if ( mIdxSelItem < 0 ) {
+    kdDebug(5006) << "KMFilterListBox::slotTop called while no filter is selected, ignoring." << endl;
+    return;
+  }
+  if ( mIdxSelItem == 0 ) {
+    kdDebug(5006) << "KMFilterListBox::slotTop called while the _topmost_ filter is selected, ignoring." << endl;
+    return;
+  }
+
+  swapFilters( mIdxSelItem, 0 );
+  enableControls();
+}
+
 void KMFilterListBox::slotUp()
 {
   if ( mIdxSelItem < 0 ) {
@@ -672,6 +899,21 @@ void KMFilterListBox::slotDown()
   enableControls();
 }
 
+void KMFilterListBox::slotBottom()
+{
+  if ( mIdxSelItem < 0 ) {
+    kdDebug(5006) << "KMFilterListBox::slotBottom called while no filter is selected, ignoring." << endl;
+    return;
+  }
+  if ( mIdxSelItem == (int)mListBox->count() - 1 ) {
+    kdDebug(5006) << "KMFilterListBox::slotBottom called while the _last_ filter is selected, ignoring." << endl;
+    return;
+  }
+
+  swapFilters( mIdxSelItem, mListBox->count()-1 );
+  enableControls();
+}
+
 void KMFilterListBox::slotRename()
 {
   if ( mIdxSelItem < 0 ) {
@@ -679,29 +921,36 @@ void KMFilterListBox::slotRename()
     return;
   }
 
-  bool okPressed = FALSE;
+  bool okPressed =  false ;
   KMFilter *filter = mFilterList.at( mIdxSelItem );
 
   // enableControls should make sure this method is
   // never called when no filter is selected.
   assert( filter );
 
+  // allow empty names - those will turn auto-naming on again
+  QValidator *validator = new QRegExpValidator( QRegExp( ".*" ), 0 );
   QString newName = KInputDialog::getText
     (
      i18n("Rename Filter"),
-     i18n("Rename filter \"%1\" to:").arg( filter->pattern()->name() ) /*label*/,
+     i18n("Rename filter \"%1\" to:\n(leave the field empty for automatic naming)")
+        .arg( filter->pattern()->name() ) /*label*/,
      filter->pattern()->name() /* initial value */,
-     &okPressed, topLevelWidget()
+     &okPressed, topLevelWidget(), 0, validator
      );
+  delete validator;
 
   if ( !okPressed ) return;
 
-  if ( newName.isEmpty() )
+  if ( newName.isEmpty() ) {
     // bait for slotUpdateFilterName to
     // use automatic naming again.
     filter->pattern()->setName( "<>" );
-  else
+    filter->setAutoNaming( true );
+  } else {
     filter->pattern()->setName( newName );
+    filter->setAutoNaming( false );
+  }
 
   slotUpdateFilterName();
 }
@@ -712,8 +961,10 @@ void KMFilterListBox::enableControls()
   bool theLast = ( mIdxSelItem >= (int)mFilterList.count() - 1 );
   bool aFilterIsSelected = ( mIdxSelItem >= 0 );
 
+  mBtnTop->setEnabled( aFilterIsSelected && !theFirst );
   mBtnUp->setEnabled( aFilterIsSelected && !theFirst );
   mBtnDown->setEnabled( aFilterIsSelected && !theLast );
+  mBtnBot->setEnabled( aFilterIsSelected && !theLast );
   mBtnCopy->setEnabled( aFilterIsSelected );
   mBtnDelete->setEnabled( aFilterIsSelected );
   mBtnRename->setEnabled( aFilterIsSelected );
@@ -722,46 +973,48 @@ void KMFilterListBox::enableControls()
     mListBox->ensureCurrentVisible();
 }
 
-void KMFilterListBox::loadFilterList()
+void KMFilterListBox::loadFilterList( bool createDummyFilter )
 {
   assert(mListBox);
-  setEnabled(FALSE);
+  setEnabled( false );
+  emit resetWidgets();
   // we don't want the insertion to
   // cause flicker in the edit widgets.
-  blockSignals(TRUE);
+  blockSignals( true );
 
   // clear both lists
   mFilterList.clear();
   mListBox->clear();
 
-	QPtrList<KMFilter> *manager;
+  const KMFilterMgr *manager = 0;
   if(bPopFilter)
-	{
+  {
     mShowLater = kmkernel->popFilterMgr()->showLaterMsgs();
-		manager = kmkernel->popFilterMgr();
-	}
-	else
-	{
-		manager = kmkernel->filterMgr();
-	}
+    manager = kmkernel->popFilterMgr();
+  }
+  else
+  {
+    manager = kmkernel->filterMgr();
+  }
+  Q_ASSERT( manager );
 
-  QPtrListIterator<KMFilter> it( *manager );
-  for ( it.toFirst() ; it.current() ; ++it ) {
+  QValueListConstIterator<KMFilter*> it;
+  for ( it = manager->filters().constBegin() ; it != manager->filters().constEnd() ; ++it ) {
     mFilterList.append( new KMFilter( **it ) ); // deep copy
     mListBox->insertItem( (*it)->pattern()->name() );
   }
 
-  blockSignals(FALSE);
-  setEnabled(TRUE);
+  blockSignals( false );
+  setEnabled( true );
 
   // create an empty filter when there's none, to avoid a completely
   // disabled dialog (usability tests indicated that the new-filter
   // button is too hard to find that way):
-  if ( !mListBox->count() )
+  if ( !mListBox->count() && createDummyFilter )
     slotNew();
 
-  assert( mListBox->count() > 0 );
-  mListBox->setSelected( 0, true );
+  if ( mListBox->count() > 0 )
+    mListBox->setSelected( 0, true );
 
   enableControls();
 }
@@ -776,17 +1029,23 @@ void KMFilterListBox::insertFilter( KMFilter* aFilter )
   if ( mIdxSelItem < 0 ) {
     // none selected -> append
     mFilterList.append( aFilter );
-    mListBox->setSelected( mListBox->count() - 1, TRUE );
+    mListBox->setSelected( mListBox->count() - 1, true );
     //    slotSelected( mListBox->count() - 1 );
   } else {
     // insert just before selected
     mFilterList.insert( mIdxSelItem, aFilter );
-    mListBox->setSelected( mIdxSelItem, TRUE );
+    mListBox->setSelected( mIdxSelItem, true );
     //    slotSelected( mIdxSelItem );
   }
 
 }
 
+void KMFilterListBox::appendFilter( KMFilter* aFilter )
+{
+    mFilterList.append( aFilter );
+    mListBox->insertItem( aFilter->pattern()->name(), -1 );
+}
+
 void KMFilterListBox::swapNeighbouringFilters( int untouchedOne, int movedOne )
 {
   // must be neighbours...
@@ -806,6 +1065,19 @@ void KMFilterListBox::swapNeighbouringFilters( int untouchedOne, int movedOne )
   mIdxSelItem += movedOne - untouchedOne;
 }
 
+void KMFilterListBox::swapFilters( int from, int to )
+{
+  QListBoxItem *item = mListBox->item( from );
+  mListBox->takeItem( item );
+  mListBox->insertItem( item, to );
+
+  KMFilter* filter = mFilterList.take( from );
+  mFilterList.insert( to, filter );
+
+  mIdxSelItem = to;
+  mListBox->setCurrentItem( mIdxSelItem );
+  mListBox->setSelected( mIdxSelItem, true );
+}
 
 //=============================================================================
 //
@@ -817,9 +1089,9 @@ KMFilterActionWidget::KMFilterActionWidget( QWidget *parent, const char* name )
   : QHBox( parent, name )
 {
   int i;
-  mActionList.setAutoDelete(TRUE);
+  mActionList.setAutoDelete( true );
 
-  mComboBox = new QComboBox( FALSE, this );
+  mComboBox = new QComboBox(  false , this );
   assert( mComboBox );
   mWidgetStack = new QWidgetStack(this);
   assert( mWidgetStack );
@@ -865,7 +1137,7 @@ KMFilterActionWidget::KMFilterActionWidget( QWidget *parent, const char* name )
 void KMFilterActionWidget::setAction( const KMFilterAction* aAction )
 {
   int i=0;
-  bool found = FALSE;
+  bool found =  false ;
   int count = mComboBox->count() - 1 ; // last entry is the empty one
   QString label = ( aAction ) ? aAction->label() : QString::null ;
 
@@ -880,7 +1152,7 @@ void KMFilterActionWidget::setAction( const KMFilterAction* aAction )
       // the combo box
       mComboBox->setCurrentItem(i); // (mm) also raise the widget, but doesn't
       mWidgetStack->raiseWidget(i);
-      found = TRUE;
+      found = true;
     } else // clear the parameter widget
       mActionList.at(i)->clearParamWidget( mWidgetStack->widget(i) );
   if ( found ) return;
@@ -933,7 +1205,7 @@ void KMFilterActionWidgetLister::setActionList( QPtrList<KMFilterAction> *aList
 
   mActionList = aList;
 
-  ((QWidget*)parent())->setEnabled( TRUE );
+  ((QWidget*)parent())->setEnabled( true );
 
   if ( aList->count() == 0 ) {
     slotClear();
@@ -967,7 +1239,7 @@ void KMFilterActionWidgetLister::reset()
 
   mActionList = 0;
   slotClear();
-  ((QWidget*)parent())->setEnabled( FALSE );
+  ((QWidget*)parent())->setEnabled(  false  );
 }
 
 QWidget* KMFilterActionWidgetLister::createWidget( QWidget *parent )
@@ -1028,11 +1300,11 @@ void KMPopFilterActionWidget::setAction( KMPopFilterAction aAction )
   blockSignals( true );
   if(!mActionMap[aAction]->isChecked())
   {
-    mActionMap[aAction]->setChecked(true);
+    mActionMap[aAction]->setChecked( true );
   }
   blockSignals( false );
 
-  setEnabled(true);
+  setEnabled( true );
 }
 
 KMPopFilterAction  KMPopFilterActionWidget::action()
@@ -1048,11 +1320,35 @@ void KMPopFilterActionWidget::slotActionClicked(int aId)
 
 void KMPopFilterActionWidget::reset()
 {
-  blockSignals(TRUE);
-  mActionMap[Down]->setChecked( TRUE );
-  blockSignals(FALSE);
+  blockSignals( true );
+  mActionMap[Down]->setChecked( true );
+  blockSignals( false );
 
-  setEnabled( FALSE );
+  setEnabled(  false  );
+}
+
+void KMFilterDlg::slotImportFilters()
+{
+    FilterImporterExporter importer( this, bPopFilter );
+    QValueList<KMFilter*> filters = importer.importFilters();
+    // FIXME message box how many were imported?
+    if (filters.isEmpty()) return;
+
+    QValueListConstIterator<KMFilter*> it;
+
+    for ( it = filters.constBegin() ; it != filters.constEnd() ; ++it ) {
+        mFilterList->appendFilter( *it ); // no need to deep copy, ownership passes to the list
+    }
+}
+
+void KMFilterDlg::slotExportFilters()
+{
+    FilterImporterExporter exporter( this, bPopFilter );
+    QValueList<KMFilter*> filters = mFilterList->filtersForSaving();
+    exporter.exportFilters( filters );
+    QValueList<KMFilter*>::iterator it;
+    for ( it = filters.begin(); it != filters.end(); ++it )
+        delete *it;
 }
 
 #include "kmfilterdlg.moc"
