@@ -43,6 +43,7 @@
 #include <stdio.h>
 #include <errno.h>
 #include <assert.h>
+#include <ctype.h>
 #include <unistd.h>
 
 #ifdef HAVE_FCNTL_H
@@ -84,12 +85,14 @@ KMFolderMbox::KMFolderMbox(KMFolder* folder, const char* name)
 //-----------------------------------------------------------------------------
 KMFolderMbox::~KMFolderMbox()
 {
-  if (mOpenCount>0) close(true);
-  if (kmkernel->undoStack()) kmkernel->undoStack()->folderDestroyed( folder() );
+  if (mOpenCount>0)
+    close("~kmfoldermbox", true);
+  if (kmkernel->undoStack())
+    kmkernel->undoStack()->folderDestroyed( folder() );
 }
 
 //-----------------------------------------------------------------------------
-int KMFolderMbox::open()
+int KMFolderMbox::open(const char *owner)
 {
   int rc = 0;
 
@@ -253,20 +256,8 @@ int KMFolderMbox::create()
 
 
 //-----------------------------------------------------------------------------
-void KMFolderMbox::close(bool aForced)
+void KMFolderMbox::reallyDoClose(const char* owner)
 {
-  if (mOpenCount <= 0 || !mStream) return;
-  if (mOpenCount > 0) mOpenCount--;
-  if (mOpenCount > 0 && !aForced) return;
-#if 0 // removed hack that prevented closing system folders (see kmail-devel discussion about mail expiring)
-  if ( (folder() != kmkernel->inboxFolder())
-        && folder()->isSystemFolder() && !aForced )
-  {
-      mOpenCount = 1;
-      return;
-  }
-#endif
-
   if (mAutoCreateIndex)
   {
       if (KMFolderIndex::IndexOk != indexStatus()) {
@@ -661,7 +652,7 @@ int KMFolderMbox::createIndexFromContents()
             int cidx = contentTypeStr.find( "charset=" );
             if ( cidx != -1 ) {
               charset = contentTypeStr.mid( cidx + 8 );
-              if ( charset[0] == '"' ) {
+              if ( !charset.isEmpty() && ( charset[0] == '"' ) ) {
                 charset = charset.mid( 1 );
               }
               cidx = 0;
@@ -826,7 +817,8 @@ KMMessage* KMFolderMbox::readMsg(int idx)
   assert(mi!=0 && !mi->isMessage());
   assert(mStream != 0);
 
-  KMMessage *msg = new KMMessage(*mi); // note that mi is deleted by the line below
+  KMMessage *msg = new KMMessage(*mi);
+  msg->setMsgInfo( mi ); // remember the KMMsgInfo object to that we can restore it when the KMMessage object is no longer needed
   mMsgList.set(idx,&msg->toMsgBase()); // done now so that the serial number can be computed
   msg->fromDwString(getDwString(idx));
   return msg;
@@ -870,12 +862,12 @@ static size_t unescapeFrom( char* str, size_t strLen ) {
 }
 
 //static
-QCString KMFolderMbox::escapeFrom( const QCString & str ) {
+QByteArray KMFolderMbox::escapeFrom( const DwString & str ) {
   const unsigned int strLen = str.length();
   if ( strLen <= STRDIM("From ") )
-    return str;
+    return KMail::Util::ByteArray( str );
   // worst case: \nFrom_\nFrom_\nFrom_... => grows to 7/6
-  QCString result( int( strLen + 5 ) / 6 * 7 + 1 );
+  QByteArray result( int( strLen + 5 ) / 6 * 7 + 1 );
 
   const char * s = str.data();
   const char * const e = s + strLen - STRDIM("From ");
@@ -908,29 +900,6 @@ QCString KMFolderMbox::escapeFrom( const QCString & str ) {
 
 #undef STRDIM
 
-//-----------------------------------------------------------------------------
-QCString& KMFolderMbox::getMsgString(int idx, QCString &mDest)
-{
-  unsigned long msgSize;
-  KMMsgInfo* mi = (KMMsgInfo*)mMsgList[idx];
-
-  assert(mi!=0);
-  assert(mStream != 0);
-
-  msgSize = mi->msgSize();
-  mDest.resize(msgSize+2);
-
-  fseek(mStream, mi->folderOffset(), SEEK_SET);
-  fread(mDest.data(), msgSize, 1, mStream);
-  mDest[msgSize] = '\0';
-
-  size_t newMsgSize = unescapeFrom( mDest.data(), msgSize );
-  newMsgSize = KMail::Util::crlf2lf( mDest.data(), newMsgSize );
-
-  return mDest;
-}
-
-
 //-----------------------------------------------------------------------------
 DwString KMFolderMbox::getDwString(int idx)
 {
@@ -960,20 +929,19 @@ DwString KMFolderMbox::getDwString(int idx)
 int KMFolderMbox::addMsg( KMMessage* aMsg, int* aIndex_ret )
 {
   if (!canAddMsgNow(aMsg, aIndex_ret)) return 0;
-  bool opened = false;
-  QCString msgText;
+  QByteArray msgText;
   char endStr[3];
   int idx = -1, rc;
   KMFolder* msgParent;
   bool editing = false;
   int growth = 0;
 
-  if (!mStream)
+  KMFolderOpener openThis(folder(), "mboxaddMsg");
+  rc = openThis.openResult();
+  if (rc)
   {
-    opened = true;
-    rc = open();
-    kdDebug(5006) << "KMFolderMBox::addMsg-open: " << rc << " of folder: " << label() << endl;
-    if (rc) return rc;
+    kdDebug(5006) << "KMFolderMbox::addMsg-open: " << rc << " of folder: " << label() << endl;
+    return rc;
   }
 
   // take message out of the folder it is currently in, if any
@@ -1018,15 +986,14 @@ if( fileD1.open( IO_WriteOnly ) ) {
     if (aMsg->headerField("Content-Type").isEmpty())  // This might be added by
       aMsg->removeHeaderField("Content-Type");        // the line above
   }
-  msgText = escapeFrom( aMsg->asString() );
-  size_t len = msgText.length();
+  msgText = escapeFrom( aMsg->asDwString() );
+  size_t len = msgText.size();
 
   assert(mStream != 0);
   clearerr(mStream);
   if (len <= 0)
   {
     kdDebug(5006) << "Message added to folder `" << name() << "' contains no data. Ignoring it." << endl;
-    if (opened) close();
     return 0;
   }
 
@@ -1051,15 +1018,12 @@ if( fileD1.open( IO_WriteOnly ) ) {
   fseek(mStream,0,SEEK_END); // this is needed on solaris and others
   int error = ferror(mStream);
   if (error)
-  {
-    if (opened) close();
     return error;
-  }
 
   QCString messageSeparator( aMsg->mboxMessageSeparator() );
   fwrite( messageSeparator.data(), messageSeparator.length(), 1, mStream );
   off_t offs = ftell(mStream);
-  fwrite(msgText, len, 1, mStream);
+  fwrite(msgText.data(), len, 1, mStream);
   if (msgText[(int)len-1]!='\n') fwrite("\n\n", 1, 2, mStream);
   fflush(mStream);
   size_t size = ftell(mStream) - offs;
@@ -1081,7 +1045,6 @@ if( fileD1.open( IO_WriteOnly ) ) {
                "(No space left on device or insufficient quota?)\n"
                "Free space and sufficient quota are required to continue safely."));
     if (busy) kmkernel->kbp()->busy();
-    if (opened) close();
     kmkernel->kbp()->idle();
     */
     return error;
@@ -1100,6 +1063,7 @@ if( fileD1.open( IO_WriteOnly ) ) {
       emit numUnreadMsgsChanged( folder() );
   }
   ++mTotalMsgs;
+  mSize = -1;
 
   if ( aMsg->attachmentState() == KMMsgAttachmentUnknown &&
        aMsg->readyToShow() )
@@ -1164,16 +1128,13 @@ if( fileD1.open( IO_WriteOnly ) ) {
              "(No space left on device or insufficient quota?)\n"
              "Free space and sufficient quota are required to continue safely."));
       if (busy) kmkernel->kbp()->busy();
-      if (opened) close();
       */
       return error;
     }
   }
 
-  // some "paper work"
   if (aIndex_ret) *aIndex_ret = idx;
   emitMsgAddedSignals(idx);
-  if (opened) close();
 
   // All streams have been flushed without errors if we arrive here
   // Return success!
@@ -1262,17 +1223,11 @@ int KMFolderMbox::compact( bool silent )
 {
   // This is called only when the user explicitely requests compaction,
   // so we don't check needsCompact.
-  int openCount = mOpenCount;
 
   KMail::MboxCompactionJob* job = new KMail::MboxCompactionJob( folder(), true /*immediate*/ );
   int rc = job->executeNow( silent );
   // Note that job autodeletes itself.
 
-  if (openCount > 0)
-  {
-    open();
-    mOpenCount = openCount;
-  }
   // If this is the current folder, the changed signal will ultimately call
   // KMHeaders::setFolderInfoStatus which will override the message, so save/restore it
   QString statusMsg = BroadcastStatus::instance()->statusMsg();
@@ -1311,5 +1266,13 @@ int KMFolderMbox::expungeContents()
   return rc;
 }
 
+//-----------------------------------------------------------------------------
+/*virtual*/
+Q_INT64 KMFolderMbox::doFolderSize() const
+{
+  QFileInfo info( location() );
+  return (Q_INT64)(info.size());
+}
+
 //-----------------------------------------------------------------------------
 #include "kmfoldermbox.moc"
