@@ -65,7 +65,7 @@ namespace KParts {
   struct URLArgs;
 }
 
-/** 
+/**
    This class implements a "reader window", that is a window
    used for reading or viewing messages.
 */
@@ -76,6 +76,7 @@ class KMReaderWin: public QWidget, public KMail::Interface::Observer {
   friend void KMMimePartTree::itemClicked( QListViewItem* item );
   friend void KMMimePartTree::itemRightClicked( QListViewItem* item, const QPoint & );
   friend void KMMimePartTree::slotSaveAs();
+  friend void KMMimePartTree::startDrag();
 
   friend class KMail::ObjectTreeParser;
   friend class KMail::KHtmlPartHtmlWriter;
@@ -127,6 +128,8 @@ public:
   /** Set the override character encoding. */
   void setOverrideEncoding( const QString & encoding );
 
+  void setPrintFont( const QFont& font );
+
   /** Get codec corresponding to the currently selected override character encoding.
       @return The override codec or 0 if auto-detection is selected. */
   const QTextCodec * overrideCodec() const;
@@ -159,6 +162,10 @@ public:
   /** Clear the reader and discard the current message. */
   void clear(bool force = false) { setMsg(0, force); }
 
+  /** Saves the relative position of the scroll view. Call this before calling update()
+      if you want to preserve the current view. */
+  void saveRelativePosition();
+
   /** Re-parse the current message. */
   void update(bool force = false);
 
@@ -244,6 +251,8 @@ public:
 
   partNode * partNodeForId( int id );
 
+  KURL tempFileUrlFromPartNode( const partNode * node );
+
   /** Returns id of message part from given URL or -1 if invalid. */
   static int msgPartFromUrl(const KURL &url);
 
@@ -275,6 +284,24 @@ public:
 
   QWidget* mainWindow() { return mMainWindow; }
 
+  /** Returns wether the message should be decryted. */
+  bool decryptMessage() const;
+
+  /** Enforce message decryption. */
+  void setDecryptMessageOverwrite( bool overwrite = true ) { mDecrytMessageOverwrite = overwrite; }
+
+  /** Show signature details. */
+  bool showSignatureDetails() const { return mShowSignatureDetails; }
+
+  /** Show signature details. */
+  void setShowSignatureDetails( bool showDetails = true ) { mShowSignatureDetails = showDetails; }
+
+  /* show or hide the list that points to the attachments */
+  bool showAttachmentQuicklist() const { return mShowAttachmentQuicklist; }
+
+  /* show or hide the list that points to the attachments */
+  void setShowAttachmentQuicklist( bool showAttachmentQuicklist = true ) { mShowAttachmentQuicklist = showAttachmentQuicklist; }
+
 signals:
   /** Emitted after parsing of a message to have it stored
       in unencrypted state in it's folder. */
@@ -321,6 +348,8 @@ public slots:
 
   /** The user selected "Find" from the menu. */
   void slotFind();
+  /** The user selected "Find Next" from the menu. */
+  void slotFindNext();
 
   /** The user toggled the "Fixed Font" flag from the view menu. */
   void slotToggleFixedFont();
@@ -351,12 +380,19 @@ public slots:
   void slotIMChat();
   void contactStatusChanged( const QString &uid);
 
-	void slotLevelQuote( int l );
+  void slotLevelQuote( int l );
+  void slotTouchMessage();
+
+  void slotDeleteAttachment( partNode* node );
+  void slotEditAttachment( partNode* node );
+
+  KMail::CSSHelper* cssHelper();
 
 protected slots:
   void slotCycleHeaderStyles();
   void slotBriefHeaders();
   void slotFancyHeaders();
+  void slotEnterpriseHeaders();
   void slotStandardHeaders();
   void slotLongHeaders();
   void slotAllHeaders();
@@ -370,7 +406,6 @@ protected slots:
   /** Some attachment operations. */
   void slotAtmView( int id, const QString& name );
   void slotDelayedResize();
-  void slotTouchMessage();
   void slotHandleAttachment( int );
 
 protected:
@@ -397,13 +432,20 @@ protected:
 
   /** Creates a nice mail header depending on the current selected
     header style. */
-  QString writeMsgHeader(KMMessage* aMsg, bool hasVCard=false);
+  QString writeMsgHeader(KMMessage* aMsg, bool hasVCard=false, bool topLevel=false);
 
   /** Writes the given message part to a temporary file and returns the
       name of this file or QString::null if writing failed.
   */
   QString writeMessagePartToTempFile( KMMessagePart* msgPart, int partNumber );
 
+  /**
+    Creates a temporary dir for saving attachments, etc.
+    Will be automatically deleted when another message is viewed.
+    @param param Optional part of the directory name.
+  */
+  QString createTempDir( const QString &param = QString() );
+
   /** show window containing infos about a vCard. */
   void showVCard(KMMessagePart *msgPart);
 
@@ -422,13 +464,13 @@ protected:
 
 private slots:
   void slotSetEncoding();
+  void injectAttachments();
 
 private:
   void adjustLayout();
   void createWidgets();
   void createActions( KActionCollection * ac );
   void saveSplitterSizes( KConfigBase & c ) const;
-  QString createAtmFileLink() const;
 
   KRadioAction * actionForHeaderStyle( const KMail::HeaderStyle *,
                                        const KMail::HeaderStrategy * );
@@ -436,6 +478,8 @@ private:
   /** Read override codec from configuration */
   void readGlobalOverrideCodec();
 
+  QString renderAttachments( partNode *node, const QColor &bgColor );
+
 private:
   bool mHtmlMail, mHtmlLoadExternal, mHtmlOverride, mHtmlLoadExtOverride;
   int mAtmCurrent;
@@ -455,7 +499,7 @@ private:
   /** where did the user save the attachment last time */
   QString mSaveAttachDir;
   static const int delay;
-  QTimer updateReaderWinTimer;
+  QTimer mUpdateReaderWinTimer;
   QTimer mResizeTimer;
   QTimer mDelayedMarkTimer;
   QString mOverrideEncoding;
@@ -492,6 +536,9 @@ private:
   unsigned long mWaitingForSerNum;
   float mSavedRelativePosition;
 	int mLevelQuote;
+  bool mDecrytMessageOverwrite;
+  bool mShowSignatureDetails;
+  bool mShowAttachmentQuicklist;
 };
 
 
