@@ -37,6 +37,7 @@
 #include <mimelib/utility.h>
 #include <qregexp.h>
 #include <kasciistricmp.h>
+#include "util.h"
 
 /*
   ===========================================================================
@@ -179,7 +180,7 @@ const QCString & partNode::encodedBody() {
     return mEncodedBody;
 
   if ( mDwPart )
-    mEncodedBody = mDwPart->AsString().c_str();
+    mEncodedBody = KMail::Util::CString( mDwPart->Body().AsString() );
   else
     mEncodedBody = 0;
   mEncodedOk = true;
@@ -340,10 +341,10 @@ KMMsgSignatureState  partNode::overallSignatureState() const
 }
 
 
-int partNode::nodeId()
+int partNode::nodeId() const
 {
     int curId = 0;
-    partNode* rootNode = this;
+    partNode* rootNode = const_cast<partNode*>( this );
     while( rootNode->mRoot )
         rootNode = rootNode->mRoot;
     return rootNode->calcNodeIdOrFindNode( curId, this, 0, 0 );
