@@ -42,7 +42,6 @@
 #include <kxmlguifactory.h>
 #include <kcolordrag.h>
 #include <kiconeffect.h>
-#include <kprinter.h>
 #include <klocale.h>
 #include <kstandarddirs.h>
 #include <kmessagebox.h>
@@ -65,6 +64,7 @@
 #include "knotealarmdlg.h"
 #include "knotehostdlg.h"
 #include "knotesnetsend.h"
+#include "knoteprinter.h"
 #include "version.h"
 
 #include "pushpin.xpm"
@@ -167,28 +167,31 @@ KNote::KNote( QDomDocument buildDoc, Journal *j, QWidget *parent, const char *na
     KXMLGUIFactory factory( &builder, this );
     factory.addClient( this );
 
-    m_menu = static_cast<KPopupMenu*>(factory.container( "note_context", this ));
-    m_edit_menu = static_cast<KPopupMenu*>(factory.container( "note_edit", this ));
-    m_tool = static_cast<KToolBar*>(factory.container( "note_tool", this ));
-    m_tool->setIconSize( 10 );
-    m_tool->setFixedHeight( 16 );
-    m_tool->setIconText( KToolBar::IconOnly );
+    m_menu = dynamic_cast<KPopupMenu*>(factory.container( "note_context", this ));
+    m_edit_menu = dynamic_cast<KPopupMenu*>(factory.container( "note_edit", this ));
+    m_tool = dynamic_cast<KToolBar*>(factory.container( "note_tool", this ));
 
-    // if there was just a way of making KComboBox adhere the toolbar height...
-    QObjectList *list = m_tool->queryList( "KComboBox" );
-    QObjectListIt it( *list );
-    while ( it.current() != 0 )
-    {
-        KComboBox *combo = (KComboBox *)it.current();
-        QFont font = combo->font();
-        font.setPointSize( 7 );
-        combo->setFont( font );
-        combo->setFixedHeight( 14 );
-        ++it;
+    if ( m_tool ) {
+      m_tool->setIconSize( 10 );
+      m_tool->setFixedHeight( 16 );
+      m_tool->setIconText( KToolBar::IconOnly );
+
+      // if there was just a way of making KComboBox adhere the toolbar height...
+      QObjectList *list = m_tool->queryList( "KComboBox" );
+      QObjectListIt it( *list );
+      while ( it.current() != 0 )
+      {
+          KComboBox *combo = (KComboBox *)it.current();
+          QFont font = combo->font();
+          font.setPointSize( 7 );
+          combo->setFont( font );
+          combo->setFixedHeight( 14 );
+          ++it;
+      }
+      delete list;
+
+      m_tool->hide();
     }
-    delete list;
-
-    m_tool->hide();
 
     setFocusProxy( m_editor );
 
@@ -413,7 +416,10 @@ void KNote::saveData()
 void KNote::saveConfig() const
 {
     m_config->setWidth( width() );
-    m_config->setHeight( height() - (m_tool->isHidden() ? 0 : m_tool->height()) );
+    if ( m_tool )
+      m_config->setHeight( height() - (m_tool->isHidden() ? 0 : m_tool->height()) );
+    else
+      m_config->setHeight( 0 );
     m_config->setPosition( pos() );
 
     NETWinInfo wm_client( qt_xdisplay(), winId(), qt_xrootwin(), NET::WMDesktop );
@@ -798,61 +804,19 @@ void KNote::slotPrint()
 {
     saveData();
 
-    KPrinter printer;
-    printer.setFullPage( true );
+    QString content;
+    if ( m_editor->textFormat() == PlainText )
+        content = QStyleSheet::convertFromPlainText( m_editor->text() );
+    else
+        content = m_editor->text();
 
-    if ( printer.setup( 0, i18n("Print %1").arg(name()) ) )
-    {
-        QPainter painter;
-        painter.begin( &printer );
-
-        const int margin = 40;  // pt
-
-        QPaintDeviceMetrics metrics( painter.device() );
-        int marginX = margin * metrics.logicalDpiX() / 72;
-        int marginY = margin * metrics.logicalDpiY() / 72;
-
-        QRect body( marginX, marginY,
-                    metrics.width() - marginX * 2,
-                    metrics.height() - marginY * 2 );
-
-        QString content;
-        if ( m_editor->textFormat() == PlainText )
-            content = QStyleSheet::convertFromPlainText( m_editor->text() );
-        else
-            content = m_editor->text();
-
-        QSimpleRichText text( content, m_config->font(), m_editor->context(),
-                              m_editor->styleSheet(), m_editor->mimeSourceFactory(),
-                              body.height() /*, linkColor, linkUnderline? */ );
-
-        text.setWidth( &painter, body.width() );
-        QRect view( body );
-
-        int page = 1;
-
-        for (;;)
-        {
-            text.draw( &painter, body.left(), body.top(), view, colorGroup() );
-            view.moveBy( 0, body.height() );
-            painter.translate( 0, -body.height() );
-
-            // page numbers
-            painter.setFont( m_config->font() );
-            painter.drawText(
-                view.right() - painter.fontMetrics().width( QString::number( page ) ),
-                view.bottom() + painter.fontMetrics().ascent() + 5, QString::number( page )
-            );
-
-            if ( view.top() >= text.height() )
-                break;
-
-            printer.newPage();
-            page++;
-        }
-
-        painter.end();
-    }
+    KNotePrinter printer;
+    printer.setMimeSourceFactory( m_editor->mimeSourceFactory() );
+    printer.setFont( m_config->font() );
+    printer.setContext( m_editor->context() );
+    printer.setStyleSheet( m_editor->styleSheet() );
+    printer.setColorGroup( colorGroup() );
+    printer.printNote( QString(), content );
 }
 
 void KNote::slotSaveAs()
@@ -1033,13 +997,13 @@ void KNote::updateFocus()
 
         if ( !m_editor->isReadOnly() )
         {
-            if ( m_tool->isHidden() && m_editor->textFormat() == QTextEdit::RichText )
+            if ( m_tool && m_tool->isHidden() && m_editor->textFormat() == QTextEdit::RichText )
             {
                 m_tool->show();
                 setGeometry( x(), y(), width(), height() + m_tool->height() );
             }
         }
-        else if ( !m_tool->isHidden() )
+        else if ( m_tool && !m_tool->isHidden() )
         {
             m_tool->hide();
             setGeometry( x(), y(), width(), height() - m_tool->height() );
@@ -1053,7 +1017,7 @@ void KNote::updateFocus()
         m_button->hide();
         m_editor->cornerWidget()->hide();
 
-        if ( !m_tool->isHidden() )
+        if ( m_tool && !m_tool->isHidden() )
         {
             m_tool->hide();
             setGeometry( x(), y(), width(), height() - m_tool->height() );
@@ -1189,22 +1153,24 @@ void KNote::updateLayout()
         QPoint( contentsRect().x(),
                 contentsRect().y() + headerHeight + s_ppOffset ),
         QPoint( contentsRect().right(),
-                contentsRect().bottom() - (m_tool->isHidden() ? 0 : m_tool->height()) )
+                contentsRect().bottom() - ( m_tool ? (m_tool->isHidden() ? 0 : m_tool->height()) : 0 ) )
     ) );
 
-    m_tool->setGeometry(
-        contentsRect().x(),
-        contentsRect().bottom() - m_tool->height() + 1,
-        contentsRect().width(),
-        m_tool->height()
-    );
+    if( m_tool ) {
+      m_tool->setGeometry(
+          contentsRect().x(),
+          contentsRect().bottom() - m_tool->height() + 1,
+          contentsRect().width(),
+          m_tool->height()
+      );
+    }
 
     if ( s_ppOffset )
         m_fold->move( width() - 15, height() - 15 );
 
     setMinimumSize(
         m_editor->cornerWidget()->width() + margin*2,
-        headerHeight + s_ppOffset + (m_tool->isHidden() ? 0 : m_tool->height()) +
+        headerHeight + s_ppOffset + ( m_tool ? (m_tool->isHidden() ? 0 : m_tool->height() ) : 0 ) +
                 m_editor->cornerWidget()->height() + margin*2
     );
 
