@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
     As a special exception, permission is given to link this program
     with any edition of Qt, and distribute the resulting executable,
@@ -55,18 +55,28 @@ class KMailPlugin : public Kontact::Plugin
     virtual bool createDCOPInterface( const QString& serviceType );
     virtual Kontact::Summary *createSummaryWidget( QWidget *parent );
     virtual QString tipFile() const;
+    int weight() const { return 200; }
 
     virtual QStringList invisibleToolbarActions() const;
     virtual bool queryClose() const;
+
+    //override
+    void loadProfile( const QString& profileDirectory );
+
+    //override
+    void saveToProfile( const QString& profileDirectory );
+
   protected:
-    virtual KParts::Part* createPart();
-    void openComposer( const KURL& );
+    virtual KParts::ReadOnlyPart* createPart();
+    void openComposer( const KURL& = KURL() );
+    void openComposer( const QString& to );
     bool canDecodeDrag( QMimeSource * );
     void processDropEvent( QDropEvent * );
 
 
   protected slots:
     void slotNewMail();
+    void slotSyncFolders();
 
   private:
     KMailIface_stub *mStub;
