@@ -28,6 +28,8 @@
 #include <qscrollview.h>
 #include <qlabel.h>
 
+#include <libkcal/calendar.h>
+
 #include "calprinter.h"
 
 #include "agendaview.h"
@@ -38,6 +40,7 @@ class QBoxLayout;
 
 class KOAgenda;
 class KOAgendaItem;
+class TimeLabels;
 class KConfig;
 
 namespace KCal {
@@ -48,50 +51,6 @@ namespace KOrg {
   class IncidenceChangerBase;
 }
 
-class TimeLabels : public QScrollView
-{
-    Q_OBJECT
-  public:
-    TimeLabels( int rows, QWidget *parent = 0, const char *name = 0,
-                WFlags f = 0 );
-
-    /** Calculates the minimum width */
-    virtual int minimumWidth() const;
-
-    /** updates widget's internal state */
-    void updateConfig();
-
-    /**  */
-    void setAgenda( KOAgenda *agenda );
-
-    /**  */
-    virtual void paintEvent( QPaintEvent *e );
-
-  public slots:
-    /** update time label positions */
-    void positionChanged();
-
-  protected:
-    void drawContents( QPainter *p, int cx, int cy, int cw, int ch );
-
-  private slots:
-    /** update the position of the marker showing the mouse position */
-    void mousePosChanged(const QPoint &pos);
-
-    void showMousePos();
-    void hideMousePos();
-
-    void setCellHeight( double height );
-
-  private:
-    int mRows;
-    double mCellHeight;
-    int mMiniWidth;
-    KOAgenda* mAgenda;
-
-    QFrame *mMousePos;  // shows a marker for the current mouse position in y direction
-};
-
 class EventIndicator : public QFrame
 {
     Q_OBJECT
@@ -110,8 +69,6 @@ class EventIndicator : public QFrame
 
   private:
     int mColumns;
-    QHBox *mTopBox;
-    QBoxLayout *mTopLayout;
     Location mLocation;
     QPixmap mPixmap;
     QMemArray<bool> mEnabled;
@@ -146,11 +103,11 @@ class KOAlternateLabel : public QLabel
   KOAgendaView is the agenda-like view used to display events in a single one or
   multi-day view.
 */
-class KOAgendaView : public KOrg::AgendaView
+class KOAgendaView : public KOrg::AgendaView, public KCal::Calendar::Observer
 {
     Q_OBJECT
   public:
-    KOAgendaView( Calendar *cal, QWidget *parent = 0, const char *name = 0 );
+    KOAgendaView( Calendar *cal, QWidget *parent = 0, const char *name = 0, bool isSideBySide = false );
     virtual ~KOAgendaView();
 
 
@@ -191,6 +148,14 @@ class KOAgendaView : public KOrg::AgendaView
     /** Show only incidences from the given resource. */
     void setResource( KCal::ResourceCalendar *res, const QString &subResource = QString::null );
 
+    KOAgenda* agenda() const { return mAgenda; }
+    QSplitter* splitter() const { return mSplitterAgenda; }
+
+    /* reimplmented from KCal::Calendar::Observer */
+    void calendarIncidenceAdded( Incidence *incidence );
+    void calendarIncidenceChanged( Incidence *incidence );
+    void calendarIncidenceRemoved( Incidence *incidence );
+
   public slots:
     virtual void updateView();
     virtual void updateConfig();
@@ -231,6 +196,9 @@ class KOAgendaView : public KOrg::AgendaView
       const Qt::Orientation orient=Qt::Horizontal );
 
     void clearTimeSpanSelection();
+
+    void resourcesChanged();
+
   signals:
     void toggleExpand();
     void zoomViewHorizontally(const QDate &, int count );
@@ -319,6 +287,9 @@ class KOAgendaView : public KOrg::AgendaView
 
     KCal::ResourceCalendar *mResource;
     QString mSubResource;
+
+    bool mIsSideBySide;
+    bool mPendingChanges;
 };
 
 #endif
