@@ -1,6 +1,7 @@
 /*
     This file is part of KOrganizer.
     Copyright (c) 2001 Cornelius Schumacher <schumacher@kde.org>
+    Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
     As a special exception, permission is given to link this program
     with any edition of Qt, and distribute the resulting executable,
@@ -47,41 +48,49 @@ class KOEditorGeneralEvent : public KOEditorGeneral
 
     void initTime(QWidget *,QBoxLayout *);
     void initClass(QWidget *,QBoxLayout *);
+    void initInvitationBar( QWidget* parent, QBoxLayout *layout );
 
     void finishSetup();
 
     /** Set widgets to default values */
-    void setDefaults( QDateTime from, QDateTime to, bool allDay );
+    void setDefaults( const QDateTime &from, const QDateTime &to, bool allDay );
     /**
       Read event object and setup widgets accordingly. If templ is true, the
       event is read as template, i.e. the time and date information isn't set.
     */
-    void readEvent( Event *, bool tmpl = false );
+    void readEvent( Event *event, Calendar *calendar, bool tmpl = false );
     /** Write event settings to event object */
     void writeEvent( Event * );
 
     /** Check if the input is valid. */
     bool validateInput();
 
+    void updateRecurrenceSummary( const QString &summary );
+
+    QFrame* invitationBar() const { return mInvitationBar; }
+
   public slots:
-    void setDateTimes( QDateTime start, QDateTime end );
+    void setDateTimes( const QDateTime &start, const QDateTime &end );
     void setDuration();
 
   protected slots:
     void timeStuffDisable( bool disable );
-    void dontAssociateTime( bool noTime );
+    void associateTime( bool time );
 
     void startTimeChanged( QTime );
-    void startDateChanged( QDate );
+    void startDateChanged( const QDate& );
     void endTimeChanged( QTime );
-    void endDateChanged( QDate );
+    void endDateChanged( const QDate& );
 
     void emitDateTimeStr();
 
   signals:
     void allDayChanged(bool);
     void dateTimeStrChanged( const QString & );
-    void dateTimesChanged( QDateTime start, QDateTime end );
+    void dateTimesChanged( const QDateTime &start, const QDateTime &end );
+    void editRecurrence();
+    void acceptInvitation();
+    void declineInvitation();
 
   private:
     QLabel                  *mStartDateLabel;
@@ -91,8 +100,10 @@ class KOEditorGeneralEvent : public KOEditorGeneral
     KTimeEdit               *mStartTimeEdit;
     KTimeEdit               *mEndTimeEdit;
     QLabel                  *mDurationLabel;
-    QCheckBox               *mNoTimeButton;
+    QCheckBox               *mAlldayEventCheckbox;
     QComboBox               *mFreeTimeCombo;
+    QLabel                  *mRecurrenceSummary;
+    QFrame                  *mInvitationBar;
 
     // current start and end date and time
     QDateTime mCurrStartDateTime;
