@@ -4,7 +4,7 @@
   Requires the Qt and KDE widget libraries, available at no cost at
   http://www.trolltech.com and http://www.kde.org respectively
 
-  Copyright (c) 2002-2004 Klarlvdalens Datakonsult AB
+  Copyright (c) 2002-2004 Klarälvdalens Datakonsult AB
         <info@klaralvdalens-datakonsult.se>
 
   This program is free software; you can redistribute it and/or modify
@@ -76,6 +76,13 @@ class KOGroupware : public QObject
                           Incidence* incidence, bool isDeleting = false,
                           bool statusChanged = false );
 
+    /**
+      Send counter proposal message.
+      @param oldEvent The original event provided in the invitations.
+      @param newEvent The new event as edited by the user.
+    */
+    void sendCounterProposal( KCal::Calendar* calendar, KCal::Event* oldEvent, KCal::Event *newEvent ) const;
+
     // THIS IS THE ACTUAL KM/KO API
     enum EventState { Accepted, ConditionallyAccepted, Declined, Request };
 
@@ -88,6 +95,8 @@ class KOGroupware : public QObject
 
     /** Updates some slot connections when the view incidence changer changes */
     void slotViewNewIncidenceChanger( IncidenceChangerBase* changer );
+
+    void initialCheckForChanges();
   protected:
     KOGroupware( CalendarView*, KCal::CalendarResources* );
 
