@@ -829,6 +829,7 @@ void KOTodoView::popupMenu( QListViewItem *item, const QPoint &, int column )
 
 void KOTodoView::newTodo()
 {
+  kdDebug() << k_funcinfo << endl;
   emit newTodoSignal( QDate::currentDate().addDays(7) );
 }
 
@@ -852,9 +853,8 @@ void KOTodoView::showTodo()
 void KOTodoView::printTodo()
 {
 #ifndef KORG_NOPRINTER
-  Calendar *cal;
   KOCoreHelper helper;
-  CalPrinter printer( this, cal, &helper );
+  CalPrinter printer( this, BaseView::calendar(), &helper );
   connect( this, SIGNAL(configChanged()), &printer, SLOT(updateConfig()) );
 
   Incidence::List selectedIncidences;
