@@ -1,11 +1,11 @@
 #ifndef _KPILOT_PILOTLOCALDATABASE_H
 #define _KPILOT_PILOTLOCALDATABASE_H
-/* pilotLocalDatabase.h			KPilot
+/* KPilot
 **
 ** Copyright (C) 1998-2001 by Dan Pilone
 ** Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
+** Copyright (C) 2006 Adriaan de Groot <groot@kde.org>
 **
-** See the .cc file for an explanation of what this file is for.
 */
 
 /*
@@ -31,6 +31,15 @@
 
 #include "pilotDatabase.h"
 
+/** @file
+* Defines the PilotLocalDatabase class, for databases stored
+* on disk (as opposed to in a handheld).
+*/
+
+/**
+* PilotLocalDatabase represents databases in the same binary format
+* as on the handheld but which are stored on local disk.
+*/
 class KDE_EXPORT PilotLocalDatabase : public PilotDatabase
 {
 public:
@@ -47,20 +56,6 @@ public:
 	*/
 	PilotLocalDatabase( const QString& path,
 		const QString& name, bool useDefaultPath=true);
-	/**
-	* Opens the local database. A default path is used
-	* ($KDEHOME/share/apps/kpilot/DBBackup)
-	* and if the file is found there, it is opened.
-	* Since a backup messes up the state of the conduits (i.e.
-	* changes on the handheld might no longer be detected after
-	* a backup run, since the conduit assumes the database to have
-	* the state of the previous conduit run,  useConduitDBs=true
-	* opens the database in $KDEHOME/share/apps/kpilot/conduits
-	*
-	* TODO: Deprecate this one.
-	*/
-	PilotLocalDatabase(const QString &name, bool useConduitDBs /* =false */);
-
 
 	/**
 	* Opens the local database. This is primarily for testing
@@ -88,8 +83,8 @@ public:
 	virtual int readAppBlock(unsigned char* buffer, int maxLen);
 	// Writes the application block info.
 	virtual int writeAppBlock(unsigned char* buffer, int len);
-	// returns the number of records in the database
-	virtual int recordCount();
+	// returns the number of records in the database, 0 if not open
+	virtual unsigned int recordCount() const;
 	// Returns a QValueList of all record ids in the database.
 	virtual QValueList<recordid_t> idList();
 	// Reads a record from database by id, returns record
@@ -152,7 +147,7 @@ public:
 	* Accessor functions for the application info block.
 	*/
 	int appInfoSize() const
-		{ if (isDBOpen()) return fAppLen; else return -1; } ;
+		{ if (isOpen()) return fAppLen; else return -1; } ;
 	char *appInfo() { return fAppInfo; } ;
 
 	const struct DBInfo &getDBInfo() const { return fDBInfo; }
@@ -160,6 +155,18 @@ public:
 
 	virtual DBType dbType() const;
 
+	/** Reads local file @p path and fills in the DBInfo
+	*   structure @p d with the DBInfo from the file.
+	*
+	*   @return @c false if d is NULL
+	*   @return @c false if the file @p path does not exist
+	*   @return @c true if reading the DBInfo succeeds
+	*
+	*   @note Relatively expensive operation, since the pilot-link
+	*         library doesn't provide a cheap way of getting this
+	*         information.
+	*/
+	static bool infoFromFile( const QString &path, DBInfo *d );
 
 protected:
 	// Changes any forward slashes to underscores
@@ -176,14 +183,17 @@ private:
 	class Private;
 	Private *d;
 
+public:
 	/**
 	* For databases opened by name only (constructor 2 -- which is the
 	* preferred one, too) try this path first before the default path.
 	* Set statically so it's shared for all local databases.
 	*/
-public:
 	static void setDBPath(const QString &);
-	static const QString *getDBPath() { return fPathBase; } ;
+	/**
+	* Accessor for the extra search path.
+	*/
+	static const QString &getDBPath() { return *fPathBase; } ;
 private:
 	static QString *fPathBase;
 };
